@@ -1,10 +1,10 @@
 // -*- c++ -*-
 //------------------------------------------------------------------------------
-// $Id: CardBox.cpp,v 1.50 2006/02/20 18:23:17 vlg Exp $
+// $Id: CardBox.cpp,v 1.58 2006/10/06 02:04:47 vlg Exp $
 //------------------------------------------------------------------------------
 //                            CardBox.cpp
 //------------------------------------------------------------------------------
-//  Copyright (c) 2004,2005 by Vladislav Grinchenko
+//  Copyright (c) 2004-2006 by Vladislav Grinchenko
 //
 //  This program is free software; you can redistribute it and/or 
 //  modify it under the terms of the GNU General Public License   
@@ -17,20 +17,118 @@
 //------------------------------------------------------------------------------
 
 #include <limits.h>				// PATH_MAX
-#include <libgen.h>				// basename(3), dirname(3)
 #include <string.h>				// strcpy(3)
 
+#if !defined (WIN32)
+#  include <libgen.h>				// basename(3), dirname(3)
+#endif
+
 #include <gtkmm/table.h>
 #include <gtkmm/eventbox.h>
+#include <gtkmm/label.h>
+#include <gtkmm/box.h>
+#include <gtkmm/checkbutton.h>
+
+#include <assa/AutoPtr.h>
+using ASSA::AutoPtr;
 
 #include "Granule.h"
 #include "CardBox.h"
-#include "Config.h"
+#include "GrappConf.h"
 #include "MainWindow.h"
 #include "CardRef.h"
 #include "Deck.h"
+#include "MyDialogWithCheckbutton.h"
 #include "Intern.h"
 
+/*------------------------------------------------------------------------------
+ * Portable replacement of libgen functions - from Earnie Boyd <earnie@us...> 
+ *------------------------------------------------------------------------------
+ */
+#if defined(WIN32)
+
+static
+char *dirname(char *path)
+{
+    static char buffer[260];
+    size_t len;
+    if (path==NULL) {
+        strcpy(buffer, ".");
+        return buffer;
+    }
+    len = strlen(path);
+    assert(len<sizeof(buffer));
+
+    if (len != 0 && path [len-1] == ASSA_DIR_SEPARATOR) {
+        --len;
+    }
+
+    while (len != 0 && path [len-1] != ASSA_DIR_SEPARATOR) {
+        --len;
+    }
+    if (len==0) {
+        strcpy(buffer, ".");
+    } 
+	else if (len==1) {
+        if (path [0] == ASSA_DIR_SEPARATOR) {
+            strcpy(buffer, ASSA_DIR_SEPARATOR_S);
+        } 
+		else {
+            strcpy(buffer, ".");
+        }
+    } else {
+        memcpy(buffer, path, len-1);
+    }
+    return buffer;
+}
+/*---------------------------------------------------------------------------*/
+static
+char *basename(char *path)
+{
+    static char buffer[260];
+    size_t len_start, len;
+    if (path==NULL) {
+        strcpy(buffer, ".");
+        return buffer;
+    }
+    len = strlen(path);
+    assert(len<sizeof(buffer));
+
+    if (len != 0 && path [len-1] == ASSA_DIR_SEPARATOR) {
+        if (len==1) {
+            strcpy(buffer, ASSA_DIR_SEPARATOR_S);
+            return buffer;
+        }
+        --len;
+    }
+    len_start = len;
+
+    while (len != 0 && path[len-1] != ASSA_DIR_SEPARATOR) {
+        --len;
+    }
+
+    if (len==0) {
+        strcpy(buffer, ".");
+    } 
+	else {
+        memcpy(buffer, path + len, len_start - len);
+    }
+
+    return buffer;
+}
+/*---------------------------------------------------------------------------*/
+static
+char *realpath(const char *path, char *resolved_path)
+{
+    char *pszFilePart;
+    if (GetFullPathNameA(path, MAXPATHLEN, resolved_path, &pszFilePart) == 0) {
+        return NULL;
+	}
+    return resolved_path;
+}
+
+#endif /* ifdef WIN32 */
+
 /*------------------------------------------------------------------------------
  * static methods
  *------------------------------------------------------------------------------
@@ -49,8 +147,8 @@ calculate_relative_path (const std::string& abs_pname_,
 	std::string ap (abs_pname_);
 	std::string af (abs_fname_);
 
-	if (ap [ap.size ()-1] != G_DIR_SEPARATOR) {
-		ap += G_DIR_SEPARATOR_S;
+	if (ap [ap.size ()-1] != ASSA_DIR_SEPARATOR) {
+		ap += ASSA_DIR_SEPARATOR_S;
 	}
 
 	/* Remove prefix similar in both string (if any)
@@ -73,7 +171,7 @@ calculate_relative_path (const std::string& abs_pname_,
 	ap_iter = ap.begin ();
 	slash = ap_iter;
 	do {
-		while (*slash != G_DIR_SEPARATOR && slash != ap.end ()) { 
+		while (*slash != ASSA_DIR_SEPARATOR && slash != ap.end ()) { 
 			slash++;
 		}
 		if (slash == ap.end ()) {
@@ -104,9 +202,10 @@ calculate_absolute_path (const std::string& abs_pname_,
 	/** For each '..' found, remove it and its preceding directory name
 		(i.e. /Words/Travel/../.. => /Words/..).
 	*/
-	while (rf.find (".." G_DIR_SEPARATOR_S) != std::string::npos) {
+	while (rf.find (".." ASSA_DIR_SEPARATOR_S) != std::string::npos) 
+	{
 		rf.erase (rf.begin (), rf.begin ()+3);
-		idx = ap.rfind (G_DIR_SEPARATOR);
+		idx = ap.rfind (ASSA_DIR_SEPARATOR);
 		if (idx != std::string::npos) {
 			ap.erase (idx);
 		}
@@ -115,7 +214,26 @@ calculate_absolute_path (const std::string& abs_pname_,
 	/** If the first character is not '/', then you don't have an absolute
 		path as you migh have expected.
 	*/
-	return (ap + G_DIR_SEPARATOR_S + rf);
+	return (ap + ASSA_DIR_SEPARATOR_S + rf);
+}
+
+static std::string
+find_pixmap_path (const char* pixmap_)
+{
+	std::string result;
+
+#if defined (WIN32)
+	result  = "../share/granule/pixmaps/";
+	result += pixmap_;
+#else
+	result  = GRAPPDATDIR;
+	result += "pixmaps/";
+	result += pixmap_;
+#endif
+
+	DL ((GRAPP,"pixmap path: \"%s\"\n", result.c_str ()));
+
+	return result;
 }
 
 
@@ -129,25 +247,35 @@ CardBox ()
 	trace_with_mask("CardBox::CardBox",GUITRACE);
 	Gtk::EventBox* evtbox = NULL;
 	int i = 0;
-	
+	Glib::ustring pixpath;
+
 	for (i = 0; i < CARD_BOX_SIZE; i++) {
 		m_cdplayers [i] = NULL;
 	}
 
 	m_drawer_empty = 
-		Gdk::Pixbuf::create_from_file(GRAPPDATADIR "/pixmaps/drawer_empty.xpm");
+		Gdk::Pixbuf::create_from_file(find_pixmap_path ("drawer_empty.png"));
+
+	DL ((GRAPP,"Success on first icon\n"));
+
 	m_drawer_one   = 
-		Gdk::Pixbuf::create_from_file(GRAPPDATADIR "/pixmaps/drawer_one.xpm");
+		Gdk::Pixbuf::create_from_file(find_pixmap_path ("drawer_one.png"));
+	DL ((GRAPP,"second icon ok\n"));
+
 	m_drawer_many  = 
-		Gdk::Pixbuf::create_from_file(GRAPPDATADIR "/pixmaps/drawer_many.xpm");
+		Gdk::Pixbuf::create_from_file(find_pixmap_path ("drawer_many.png"));
+	DL ((GRAPP,"third icon ok\n"));
+
 	m_sched_green  = 
-		Gdk::Pixbuf::create_from_file(GRAPPDATADIR "/pixmaps/sched_green.xpm");
+		Gdk::Pixbuf::create_from_file(find_pixmap_path ("sched_green.png"));
 	m_sched_grey   =
-		Gdk::Pixbuf::create_from_file(GRAPPDATADIR "/pixmaps/sched_grey.xpm");
+		Gdk::Pixbuf::create_from_file(find_pixmap_path ("sched_grey.png"));
 	m_sched_red    = 
-		Gdk::Pixbuf::create_from_file(GRAPPDATADIR "/pixmaps/sched_red.xpm");
+		Gdk::Pixbuf::create_from_file(find_pixmap_path ("sched_red.png"));
 	m_sched_yellow = 
-		Gdk::Pixbuf::create_from_file(GRAPPDATADIR "/pixmaps/sched_yellow.xpm");
+		Gdk::Pixbuf::create_from_file(find_pixmap_path ("sched_yellow.png"));
+
+	DL ((GRAPP,"last icon ok\n"));
 
 	for (i = 0; i < CARD_BOX_SIZE; i++) {
 		m_cbs.push_back (CardBoxIF ());
@@ -195,7 +323,7 @@ CardBox ()
 	list_targets.push_back (Gtk::TargetEntry ("STRING"));
 	list_targets.push_back (Gtk::TargetEntry ("text/plain"));
 
-	for (i = 0; i < 5; i++) {
+	for (i = 0; i < CARD_BOX_SIZE; i++) {
 		evtbox = manage (new Gtk::EventBox);
 		evtbox->add (*(m_cbs [i].m_box_pic));
 		evtbox->set_events (Gdk::BUTTON_PRESS_MASK);
@@ -210,7 +338,7 @@ CardBox ()
 		cbt->attach (*evtbox, 1,2,i,i+1, Gtk::FILL, Gtk::FILL, 0,0);
 	}
 
-	for (i = 0; i < 5; i++) {
+	for (i = 0; i < CARD_BOX_SIZE; i++) {
 		cbt->attach (*m_cbs[i].m_total_count, 
 					 2,3,i,i+1, 
 					 Gtk::FILL,
@@ -218,7 +346,7 @@ CardBox ()
 					 0,0);
 	}
 
-	for (i = 0; i < 5; i++) {
+	for (i = 0; i < CARD_BOX_SIZE; i++) {
 		cbt->attach (*m_cbs [i].m_sched_vbox, 0,1,i,i+1, 
 					 Gtk::FILL, Gtk::FILL,4,0);
 	}
@@ -326,7 +454,8 @@ load (const string& fname_)
  		msg += fname_ + _("\nReason: invalid file syntax/DTD");
  		Gtk::MessageDialog e (msg, false, MESSAGE_ERROR);
  		e.run ();
-		DL ((ASSA::ERROR,"Failed to parse file: %s\n", error_msg.c_str ()));
+		DL ((ASSA::ASSAERR,
+			 "Failed to parse file: %s\n", error_msg.c_str ()));
 		ret = -1;
 	}
 
@@ -340,8 +469,8 @@ parse_xml_tree (const xmlDocPtr parser_, string error_msg_)
 {
 	trace_with_mask("CardBox::parse_xml_tree",GUITRACE);
 
-	char total_count [32];
-	char xpath [64];
+	char     total_count [32];
+	char     xpath [64];
 	xmlChar* result;
 
 	string   attr_node;
@@ -350,13 +479,26 @@ parse_xml_tree (const xmlDocPtr parser_, string error_msg_)
 	string   deck_path;			// deck path as recorded in CDF file
 	int      ccount;			// card counter
 	int      cloaded;			// count of cards loaded
-	Card*    cardptr;
 	CardRef* card_ref;
+	bool     reversed;			// Is it a backward-oriented card?
 
 	for (int i = 0; i < CARD_BOX_SIZE; i++) 
 	{
 		DL((GRAPP,"Filling Card Deck # %d ...\n", i+1));
 		m_card_deck [i].set_name (i);
+
+		/** Parse carddeck side 
+		 */
+		sprintf(xpath, "/cardfile/carddeck[%d]/@side", i+1);
+		result = Granule::get_node_by_xpath (parser_, xpath);
+		if (result != NULL) {
+			if (!strcmp ((const char*) result, "back")) {
+				m_card_deck [i].set_side_selection (BACK);
+				DL ((GRAPP,"CardDeck[%d] selected side set to BACK\n", i+1));
+			}
+			xmlFree (result);
+		}
+
 		cloaded = 0;
 		for (ccount = 1; ccount < MAX_CARDS; ccount++) 
 		{
@@ -382,6 +524,17 @@ parse_xml_tree (const xmlDocPtr parser_, string error_msg_)
 			card_id = ::atol ((attr_node.substr (1)).c_str ());
 			xmlFree (result);
 
+			/** Test to see if this reference of a reversed card.
+			 */
+			reversed = false;
+			sprintf (xpath, "/cardfile/carddeck[%d]/card[%d]/@reverse", 
+					 i+1, ccount);
+			result = Granule::get_node_by_xpath (parser_, xpath);
+			if (result != NULL) {
+				reversed = true;
+				xmlFree (result);
+			}
+
 			/** Parse the expiration time of the card (expdate tag). If
 				tag is not found, then we are dealing with an older file.
 			 */
@@ -408,7 +561,7 @@ parse_xml_tree (const xmlDocPtr parser_, string error_msg_)
 			deck_path = Glib::locale_from_utf8 ((const char*) result);
 			xmlFree (result);
 
-			if (deck_path[0] != G_DIR_SEPARATOR) {
+			if (deck_path[0] != ASSA_DIR_SEPARATOR) {
 				abs_path = calculate_absolute_path (CONFIG->get_proj_path (), 
 													deck_path);
 			}
@@ -419,8 +572,8 @@ parse_xml_tree (const xmlDocPtr parser_, string error_msg_)
 			DL((GRAPP,"Card parsed ID=%ld,\n path = \"%s\"\n", 
 				card_id, deck_path.c_str ()));
 
-			/** Look up the card in the list of decks by (ID, PATH) pair.
-				If deck is found, it is loaded (if it is not already in the
+			/** Look up the card in the list of Decks by (ID, PATH) pair.
+				If Deck is found, it is loaded (if it is not already in the
 				memory) and a newly created CardRef object is returned.
 			 */
 			card_ref = DECKMGR->lookup_card (card_id, abs_path);
@@ -430,6 +583,7 @@ parse_xml_tree (const xmlDocPtr parser_, string error_msg_)
 				if (card_ref->is_expired ()) {
 					m_cbs [i].add_one ();
 				}
+				card_ref->set_reversed (reversed);
 				m_card_deck [i].push_back (card_ref);
 				cloaded++;
 			}
@@ -455,14 +609,18 @@ save ()
 
 	static const char dtd_url[] = "http://granule.sourceforge.net/cardfile.dtd";
 
-	std::string abs_fname;
 	const std::string proj_path (CONFIG->get_proj_path ());
-	std::string deck_fname;
-	VDeck::cardlist_iterator citer;
-	CardRef* card_ref = NULL;
-	int ret = 0;
+	std::string       abs_fname;
+	std::string       deck_fname;
 
-	xmlTextWriterPtr writer;
+	VDeck::cardlist_iterator citer;
+	CardRef*          card_ref = NULL;
+	xmlTextWriterPtr  writer;
+	int               ret = 0;
+
+	if (m_fname.length () == 0) {
+		return ret;
+	}
 
 	DL((GRAPP,"Saving to \"%s\"\n", m_fname.c_str ()));
 	::unlink (m_fname.c_str ());
@@ -485,16 +643,28 @@ save ()
 
 	ret = xmlTextWriterStartElement (writer, BAD_CAST "cardfile");
 
-	for (int i = 0; i < CARD_BOX_SIZE; i++) {
+	for (int i = 0; i < CARD_BOX_SIZE; i++) 
+	{
 		ret = xmlTextWriterStartElement (writer, BAD_CAST "carddeck");
+		ret = xmlTextWriterWriteAttribute (writer, BAD_CAST "side",
+		   (m_card_deck [i].get_side_selection () == FRONT ? 
+			BAD_CAST "front" : BAD_CAST "back"));
+
 		citer = m_card_deck [i].begin ();
 
-		while (citer != m_card_deck [i].end ()) {
-
-			if ((card_ref = dynamic_cast<CardRef*> (*citer)) != NULL) {
+		while (citer != m_card_deck [i].end ()) 
+		{
+			if ((card_ref = dynamic_cast<CardRef*> (*citer)) != NULL) 
+			{
 				ret = xmlTextWriterStartElement (writer, BAD_CAST "card");
 				ret = xmlTextWriterWriteAttribute (writer, BAD_CAST "id",
-						   BAD_CAST (*citer)->get_id_str ().c_str ());
+						   BAD_CAST card_ref->get_id_str ().c_str ());
+
+				if (card_ref->get_reversed ()) {
+					xmlTextWriterWriteAttribute (writer, 
+												 BAD_CAST "reverse", 
+												 BAD_CAST "1");
+				}
 
 				abs_fname = card_ref->get_deck ().get_name ();
 				if (CONFIG->with_relpaths ()) {
@@ -535,24 +705,29 @@ save_as (const string& fname_)
 		m_fname += ext;
     }
 
-	if (Glib::file_test (fname_, Glib::FILE_TEST_IS_REGULAR)) {
+	if (Glib::file_test (fname_, Glib::FILE_TEST_IS_REGULAR)) 
+	{
 		string msg = N_("File \n") + fname_ 
 			+ _("\nalready exists.\nWould you like to delete it first?");
-		/* Gtk::MessageDialog qm (*MAINWIN,  */
+
 		Gtk::MessageDialog qm (msg, 
 							   false,
 							   Gtk::MESSAGE_QUESTION, 
 							   Gtk::BUTTONS_NONE,
 							   true);
-		qm.add_button (_("Yes"), Gtk::RESPONSE_YES);
-		qm.add_button (_("No"),  Gtk::RESPONSE_NO );
+		qm.add_button (Gtk::Stock::NO,  Gtk::RESPONSE_NO );
+		qm.add_button (Gtk::Stock::YES, Gtk::RESPONSE_YES);
+
 		if (qm.run () == Gtk::RESPONSE_NO) {
 			return -1;
 		}
 		::unlink (fname_.c_str ());
 	}
+
 	CONFIG->set_proj_name (fname_);
 	save ();
+
+	return 0;
 }
 
 void
@@ -561,7 +736,8 @@ close ()
 {
 	trace_with_mask("CardBox::close",GUITRACE);
 
-	for (int i = 0; i < CARD_BOX_SIZE; i++) {
+	for (int i = 0; i < CARD_BOX_SIZE; i++) 
+	{
 		delete m_cdplayers [i];
 		m_cdplayers [i] = NULL;
 		m_card_deck [i].clear ();
@@ -587,6 +763,7 @@ play_deck (int idx_)
 	if (m_card_deck [idx_].size ()) 
 	{
 		m_card_deck [idx_].reset_progress ();
+		m_cdplayers [idx_]->update_geometry ();
 		m_cdplayers [idx_]->run ();
 		update_count (idx_);	// User might have deleted some cards.
 	}
@@ -606,6 +783,9 @@ CardBox::
 add_deck_to_box (int idx_, Deck* from_deck_)
 {
 	trace_with_mask("CardBox::add_deck_to_box",GUITRACE);
+
+	bool with_reverse;
+
 	Assure_exit (idx_ < CARD_BOX_SIZE);
 	Assure_exit (from_deck_);
 
@@ -615,51 +795,146 @@ add_deck_to_box (int idx_, Deck* from_deck_)
 		em.run ();
 		return;
 	}
-	
-	CardRef* card_ref;
+
+	{
+		std::ostringstream os;
+
+		os << "Do you want to proceed with adding Deck\n" 
+		   << "\"" << from_deck_->get_name () << "\"\n"
+		   << " to CardBox " << (idx_ + 1) << " ?";
+
+		MyDialogWithCheckbutton qm (_(os.str ().c_str ()),
+						(_("If enabled, this option makes adding a card \n"
+						"from the Deck to a CardBox insert two references -\n"
+						"one with the front/back and another with \n"
+						"the opposite back/front orientation.")));
+
+		if (qm.run () == Gtk::RESPONSE_NO) {
+			return;
+		}
+		with_reverse = qm.get_checkbutton_status ();
+	}
+
+	DL((GRAPP,"Adding Deck \"%s\" to CardBox # %d\n", 
+		from_deck_->get_name ().c_str (), idx_));
+
+	/** Now, proceede with adding
+	 */
 	VDeck::cardlist_const_iterator iter = from_deck_->begin ();
 
-	while (iter != from_deck_->end ()) {
-		card_ref = new CardRef (**iter, *from_deck_);
+	while (iter != from_deck_->end ()) 
+	{
+		AutoPtr<CardRef> card_ref (new CardRef (**iter, *from_deck_));
 		card_ref->set_expiration (CONFIG->get_expiration (idx_));
-		m_card_deck [idx_].push_back (card_ref);
+		card_ref->set_reversed (false);
+
+		if (!find_in_all_carddecks (card_ref.get ())) {
+			m_card_deck [idx_].push_back (card_ref.release ());
+		}
+
+		if (with_reverse) {
+			AutoPtr<CardRef> reverse_card_ref(new CardRef(**iter, *from_deck_));
+			reverse_card_ref->set_expiration (CONFIG->get_expiration (idx_));
+			reverse_card_ref->set_reversed (true);
+			if (!find_in_all_carddecks (reverse_card_ref.get ())) {
+				m_card_deck [idx_].push_back (reverse_card_ref.release ());
+			}
+		}
+
 		iter++;
 	}
-	if (CONFIG->get_remove_duplicates ()) {
-		m_card_deck [idx_].remove_duplicates ();
-	}
+
+//	remove_all_duplicates (idx_);
 	m_card_deck [idx_].reset_progress ();
 	update_count (idx_);
 	m_cdplayers [idx_]->reset ();
-	MAINWIN->go_state (Dirty);
+	mark_as_modified ();
 }
 
 /**
-   Add card from DeckView to the CardBox. This function can be called 
-   both from DeckView to add selected card to CardBox #1 or when 
-   a new card is created and the configuration option 
-   { new_card_to_cardbox } is set to true.
+   Add card(s) from DeckView to the CardBox. Adding of multiple cards
+   is allowed as of v.1.2.2.
 
-   @param card_ref_ New card reference to add. Assume ownership.
+   This function can be called both from DeckView to add selected card(s) 
+   to CardBox #1, or from DeckList to add an entire deck either with
+   drag-n-drop operation or with `insert' user action from the right-click 
+   popup menu of the DeckList.
+
+   Either way, if `remove duplicates' option is activated in Preferences,
+   we ensure the uniqueness of the cards across all CardDecks. The policy
+   is that if a card is already present in any of the CardDecks, then it won't
+   be inserted again.
+
+   This function assumes the ownership of CardRef objects passed on to
+   it by the caller via card_ref_ list argument.
+
+   @param card_ref_ New card reference to add. 
+   @return 0 on success; -1 if CardFile is not open.
 */
-void 
+int
 CardBox::
-add_card_to_box (CardRef* card_ref_)
+add_cards_to_box (std::vector<CardRef*>& cardref_list_)
 {
 	trace_with_mask("CardBox::add_card_to_box",GUITRACE);
 
+	bool with_reverse;
+
 	if (m_fname.empty ()) {
 		Gtk::MessageDialog em (_("First, open or create a Card File."), 
 							   false, Gtk::MESSAGE_ERROR);
 		em.run ();
-		return;
+		return -1;
 	}
-	card_ref_->set_expiration (CONFIG->get_expiration (0));
-	m_card_deck [0].push_back (card_ref_);
-	m_card_deck [0].reset_progress ();
+
+	{
+		MyDialogWithCheckbutton qm (_("Add card(s) to the CardBox?"),
+					 (_("If enabled, this option makes adding a card \n"
+						"from the Deck to a CardBox insert two references -\n"
+						"one with the front/back and another with \n"
+						"the opposite back/front orientation.")));
+
+		if (qm.run () == Gtk::RESPONSE_NO) {
+			return -1;
+		}
+		with_reverse = qm.get_checkbutton_status ();
+	}
+	
+	bool card_added = false;
+	CardRef* card_ref;
+	std::vector<CardRef*>::iterator iter = cardref_list_.begin ();
+
+	while (iter != cardref_list_.end ()) 
+	{
+		card_ref = *iter;
+		if (!find_in_all_carddecks (card_ref)) {
+			card_ref->set_expiration (CONFIG->get_expiration (0));
+			m_card_deck [0].push_back (card_ref);
+			card_added = true;
+		}
+		if (with_reverse) {
+			AutoPtr<CardRef> reverse_card_ref (
+				new CardRef (card_ref->get_card (), card_ref->get_deck ()));
+			reverse_card_ref->set_reversed (true);
+			if (!find_in_all_carddecks (reverse_card_ref.get ())) {
+				reverse_card_ref->set_expiration (CONFIG->get_expiration (0));
+				m_card_deck [0].push_back (reverse_card_ref.release ());
+			}
+		}
+		if (!card_added) {
+			delete card_ref;
+		}
+
+		card_added = false;
+		iter++;
+	}
+
+	remove_all_duplicates (0);
 	update_count (0);
+	m_card_deck [0].reset_progress ();
 	m_cdplayers [0]->reset ();
-	MAINWIN->go_state (Dirty);
+	mark_as_modified ();
+	
+	return 0;
 }
 
 /** Move a card from one Card Box to another.
@@ -691,9 +966,14 @@ move_card_to_box (VDeck::cardlist_iterator card_ref_,
 
 	if (is_expired_) {
 		m_cbs [from_deck_].m_expired_count--;
+		if (m_cbs [from_deck_].m_expired_count < 0) {
+			m_cbs [from_deck_].m_expired_count = 0;
+		}
 		set_expiration_light (from_deck_);
 		if (m_cbs [from_deck_].m_expired_count == 0) {
-			Gtk::MessageDialog msg (_("You have learnt all\nexpired cards in this deck."), false, MESSAGE_INFO);
+			Gtk::MessageDialog msg (_("You have learnt all\n"
+									  "expired cards in this deck."), 
+									false, MESSAGE_INFO);
 			msg.run ();
 		}
 	}
@@ -724,7 +1004,7 @@ move_card_to_box (VDeck::cardlist_iterator card_ref_,
 	update_count (to_deck_);
 	set_expiration_light (to_deck_);
 
-	MAINWIN->go_state (Dirty);
+	mark_as_modified ();
 }
 
 bool 
@@ -800,35 +1080,14 @@ on_deck_dropped_cb (const Glib::RefPtr<Gdk::DragContext>& context_,
 					guint drop_timestamp_,
 					int card_deck_index_)
 {
-	std::ostringstream os;
-	Deck* src_deck;
-
 	trace_with_mask("CardBox::on_deck_dropped_cd",GUITRACE);
+
+	Deck* src_deck = NULL;
 	
     /** Doesn't work - I get bogus pointer back! (investigate)
 	    Deck* src_deck = (Deck*)(selection_data_.get_data ());
 	*/
-
 	src_deck = MAINWIN->get_dnd_source ();
-
-	os << "Proceed with adding Deck \n\"" << src_deck->get_name () << "\"\n"
-	   << " to CardBox #" << (card_deck_index_+1) << " ?";
-
-	/* Gtk::MessageDialog qm (*MAINWIN, */
-	Gtk::MessageDialog qm (_ (os.str ().c_str ()),
-						   false,
-						   Gtk::MESSAGE_QUESTION, 
-						   Gtk::BUTTONS_NONE,
-						   true);
-	qm.add_button (_("Yes"), Gtk::RESPONSE_YES);
-	qm.add_button (_("No"),  Gtk::RESPONSE_NO );
-	if (qm.run () == Gtk::RESPONSE_NO) {
-		return;
-	}
-
-	DL((GRAPP,"Adding Deck \"%s\" to CardBox # %d\n", 
-		src_deck->get_name ().c_str (), card_deck_index_));
-		
 	add_deck_to_box (card_deck_index_, src_deck);
 	context_->drag_finish (false, false, drop_timestamp_);
 }
@@ -856,3 +1115,64 @@ export_sound_bits (const std::string& path_)
 		m_card_deck [i].export_sound_bits (path_);
 	}
 }
+
+void
+CardBox::
+mark_as_modified () 
+{ 
+	MAINWIN->go_state (Dirty); 
+}
+
+
+/**
+ * Remove duplicate cards. The cards were just added to idx_ CardDeck.
+ * If a card added already exists in some other CardDeck then remove it.
+ */
+void
+CardBox::
+remove_all_duplicates (int idx_)
+{
+	trace_with_mask("CardBox::remove_all_duplicates",GUITRACE);
+
+	if (CONFIG->get_remove_duplicates ()) 
+	{
+		for (int idx = 0; idx < CARD_BOX_SIZE; idx++) {
+			m_card_deck [idx].remove_duplicates ();
+		}	
+	}
+}
+
+bool
+CardBox::
+find_in_all_carddecks (CardRef* new_card_)
+{
+	trace_with_mask("CardBox::find_in_all_carddecks",GUITRACE);
+
+	VDeck::cardlist_iterator iter;
+	bool ret = false;
+	CardRef* vcardp = NULL;
+
+	DL ((GRAPP,"Searching for \"%s\" in all CardDecks.\n",
+		 new_card_->get_question ().c_str ()));
+
+	for (int idx = 0; idx < CARD_BOX_SIZE; idx++) 
+	{
+		iter = m_card_deck [idx].begin ();
+		while (iter != m_card_deck [idx].end ()) {
+			vcardp = dynamic_cast<CardRef*> (*iter);
+			if (vcardp && 
+				new_card_->get_id () == vcardp->get_id () &&
+				new_card_->get_reversed () == vcardp->get_reversed ())
+			{
+				DL ((GRAPP, "Found matching card (\"%s\") - "
+					 "exists in deck #%d.\n", 
+					 vcardp->get_question ().c_str (), idx+1));
+				ret = true;
+				break;
+			}
+			iter++;
+		}
+	}
+
+	return ret;
+}
