@@ -2,7 +2,7 @@
 //------------------------------------------------------------------------------
 //                              CardDeck.cpp
 //------------------------------------------------------------------------------
-// $Id: CardDeck.cpp,v 1.20 2006/02/20 18:23:17 vlg Exp $
+// $Id: CardDeck.cpp,v 1.25 2006/09/30 20:42:27 vlg Exp $
 //
 //  Copyright (c) 2004,2006 by Vladislav Grinchenko
 //
@@ -14,16 +14,17 @@
 // Date: Apr 24 2004
 //------------------------------------------------------------------------------
 
+#include "Granule.h"
+#include "CardDeck.h"
+#include "Granule-main.h"
+#include "Intern.h"
+
 #include <sstream>
 #include <fstream>
 #include <algorithm>
 
 #include <glib.h>
 
-#include "CardDeck.h"
-#include "Granule-main.h"
-#include "Granule.h"
-#include "Intern.h"
 
 static const char* box_names [] = 
 { 
@@ -48,7 +49,9 @@ get_index () const
 			return (idx);
 		}
 	}
+	DL ((GRAPP,"EXCEPTIONAL CONDITION - aborting!\n"));
 	Assure_exit (false);
+	return -1;
 }
 
 bool
@@ -58,12 +61,16 @@ erase (VCard* vcard_)
 	trace_with_mask("CardDeck::erase(VCard)",GUITRACE|DECK);
 
 	cardlist_iterator p = m_cards.begin ();
-	while (p != m_cards.end ()) {
+
+	while (p != m_cards.end ()) 
+	{
 		if ((*p) == vcard_) {
 			erase (p);
+			return true;
 		}
 		p++;
 	}
+	return false;
 }
 
 void 
@@ -84,15 +91,27 @@ remove_duplicates ()
 	CardRef* cr_p;
 	CardRef* cr_q;
 
+	cardlist_iterator p;
+	cardlist_iterator q;
+
+	DL ((GRAPP,"cards before = %d\n", m_cards.size ()));
 	std::sort (m_cards.begin (), m_cards.end (), Compare_CardRefs ());
 
-	cardlist_iterator p = m_cards.begin ();
-	cardlist_iterator q = p + 1;
-	while (q != m_cards.end ()) {
+	p = m_cards.begin ();
+	if (p == m_cards.end ()) {
+		goto done;
+	}
+	q = p + 1;
+
+	while (q != m_cards.end ()) 
+	{
 		cr_p = dynamic_cast<CardRef*> (*p);
 		cr_q = dynamic_cast<CardRef*> (*q);
 		Assure_exit (cr_p && cr_q);
-		if (cr_p->get_card ().get_id () == cr_q->get_card ().get_id ()) {
+
+		if (cr_p->get_card().get_id() == cr_q->get_card().get_id() &&
+			cr_p->get_reversed() == cr_q->get_reversed ())
+		{
 			delete (*q);
 			m_cards.erase (q);
 			q = p;
@@ -102,6 +121,8 @@ remove_duplicates ()
 		}
 		q++;
 	}
+  done:
+	DL ((GRAPP,"cards after = %d\n", m_cards.size ()));
 }
 
 /** 
@@ -239,12 +260,12 @@ export_sound_bits (const std::string& path_)
 {
 	trace_with_mask("CardDeck::export_sound_bits",GUITRACE);
 
+#if !defined (WIN32)
 	string word;
 	string to;
 	string from;
 	string from_path;
 	vector<string> v;
-	struct stat st;
 	
 	from_path = Glib::filename_to_utf8 (CONFIG->get_snd_archive_path ());
 	from_path += G_DIR_SEPARATOR_S;
@@ -264,8 +285,13 @@ export_sound_bits (const std::string& path_)
 		}
 
 		to = path_ + G_DIR_SEPARATOR_S + word[0];
-		if (!g_file_test (to.c_str (), G_FILE_TEST_IS_DIR)) {
+		if (!g_file_test (to.c_str (), G_FILE_TEST_IS_DIR)) 
+		{
+#if defined (WIN32)
+			::mkdir (to.c_str ());
+#else
 			::mkdir (to.c_str (), 0755);
+#endif
 		}
 		to += G_DIR_SEPARATOR_S + word + ".wav";
 
@@ -281,4 +307,5 @@ export_sound_bits (const std::string& path_)
 
 		citer++;
 	}
+#endif	// !def WIN32
 }
