@@ -2,7 +2,7 @@
 //------------------------------------------------------------------------------
 //                              CardDeck.h
 //------------------------------------------------------------------------------
-// $Id: CardDeck.h,v 1.18 2006/02/20 18:23:17 vlg Exp $
+// $Id: CardDeck.h,v 1.20 2006/08/30 12:22:35 vlg Exp $
 //
 //  Copyright (c) 2004,2006 by Vladislav Grinchenko
 //
@@ -18,10 +18,14 @@
 
 #include "CardRef.h"
 
+using ASSA::ASSAERR;
+
 class VCard;
 
 /** To remove duplicates we compare Card References by the IDs of 
-	the cards they point to.
+	the cards they point to. If two cards happen to have the same
+	ID (case of reversed *sticky* duplicates), we arbitrarily
+	nominate lefthand-side be smaller then righthand-side.
 */
 class Compare_CardRefs {
 public:
@@ -30,6 +34,9 @@ public:
 		CardRef* cr1 = dynamic_cast<CardRef*> (vc1_);
 		CardRef* cr2 = dynamic_cast<CardRef*> (vc2_);
 		Assure_exit (cr1 && cr2);
+		if (cr1->get_card ().get_id () == cr2->get_card ().get_id ()) {
+			return (false);
+		}
 		return (cr1->get_card ().get_id () < cr2->get_card ().get_id ());
 	}
 };
@@ -44,8 +51,8 @@ public:
 
 /** 
  * CardDeck represents each of the Card Boxes on the left of the MainWindow.
- * Each CardDeck holds a list of virtual cards, CardRef, that points
- * to the real Card in the real Deck.
+ * Each CardDeck holds a list of virtual cards, CardRef, each of which points
+ * to the real Card in a real Deck.
  *
  * Because it is derived from VDeck, CardDeck can be played by DeckViewer.
  */
@@ -56,13 +63,13 @@ public:
 	CardDeck ();
 	virtual ~CardDeck ();
 
-	virtual cardlist_const_iterator begin() const { return m_cards.begin(); }
-	virtual cardlist_const_iterator end() const { return m_cards.end (); }
+	virtual cardlist_const_iterator begin () const { return m_cards.begin(); }
+	virtual cardlist_const_iterator end   () const { return m_cards.end ();  }
 
 	virtual cardlist_iterator begin () { return m_cards.begin (); }
-	virtual cardlist_iterator end   () { return m_cards.end (); }
+	virtual cardlist_iterator end   () { return m_cards.end ();   }
 
-	virtual size_t size  () const { return m_cards.size (); }
+	virtual size_t size  () const { return m_cards.size ();  }
 	virtual bool   empty () const { return m_cards.empty (); }
 
 	virtual void push_back (VCard* card_);
@@ -86,12 +93,15 @@ public:
 	*/
 	void sort_by_expiration_time ();
 
+	virtual SideSelection get_side_selection ()    { return m_side_selection; }
+	virtual void set_side_selection (SideSelection s_) { m_side_selection=s_; }
+
 	virtual void play_card (VDeck::cardlist_iterator& iter_);
 
 	virtual const string&  get_name () const { return m_cdname; }
 
-	virtual int  get_index () const;
-	virtual void get_progress (float& pct_done_, std::string& msg_, int idx_);
+	virtual int  get_index      () const;
+	virtual void get_progress   (float& pct_done_, std::string& msg_, int idx_);
 	virtual void reset_progress ();
 
 	void set_name (int idx_);
@@ -114,17 +124,20 @@ private:
 	string         m_cdname;	// Number of the card box this deck represents
 	cardlist_type  m_cards;
 
-    /// The size at the moment when DeckPlayer starts playing the deck.
+    /// The size at the moment when DeckPlayer started playing the Deck.
 	u_int          m_init_size;	
 
-	/// How many cards has already been played by the DeckPlayer so far.
+	/// How many cards has been already played by the DeckPlayer so far.
 	u_int          m_progress_idx;
-	
 };
 
+//------Inlines ---------------------------------------------------------------
 inline
 CardDeck::
-CardDeck () : m_init_size (0), m_progress_idx (0)
+CardDeck () 
+	: 
+	m_init_size      (0), 
+	m_progress_idx   (0)
 {
 	trace_with_mask("CardDeck::CardDeck",GUITRACE|DECK);
 }
