@@ -2,7 +2,7 @@
 //------------------------------------------------------------------------------
 //                              CardView.cpp
 //------------------------------------------------------------------------------
-// $Id: CardView.cpp,v 1.33 2006/03/19 15:39:44 vlg Exp $
+// $Id: CardView.cpp,v 1.37 2006/09/30 20:42:27 vlg Exp $
 //
 //  Copyright (c) 2004-2005 by Vladislav Grinchenko
 //
@@ -26,7 +26,7 @@
 
 #include "Granule.h"
 #include "MainWindow.h"
-#include "Config.h"
+#include "GrappConf.h"
 #include "CardView.h"
 #include "DeckView.h"
 #include "Card.h"
@@ -34,22 +34,53 @@
 #include "EditControls.h"
 #include "Intern.h"
 
+#define SET_CV_LABEL(l) \
+	l->set_alignment  (0.5,0.5);  \
+	l->set_padding    (0,0);      \
+	l->set_justify    (Gtk::JUSTIFY_LEFT); \
+	l->set_line_wrap  (false);    \
+	l->set_use_markup (false);    \
+	l->set_selectable (false);   
+
+#define SET_CV_TEXT(t) \
+	t->set_flags (Gtk::CAN_FOCUS);   \
+	t->set_border_width (3);         \
+	t->set_editable (true);          \
+	t->set_cursor_visible (true);    \
+	t->set_pixels_above_lines (3);   \
+	t->set_pixels_below_lines (0);   \
+	t->set_pixels_inside_wrap (0);   \
+	t->set_left_margin  (4);         \
+	t->set_right_margin (4);         \
+	t->set_indent (0);               \
+	t->set_wrap_mode (Gtk::WRAP_WORD);         \
+	t->set_justification (Gtk::JUSTIFY_LEFT);  \
+	t->set_accepts_tab (false);
+
+
 CardView::
-CardView (VCard* card_) : 
+CardView (VCard* card_) 
+	: 
+
 #ifdef IS_HILDON
 	Hildon::AppView ("Card View"),
 #endif
-	m_card (card_),
+
+	m_card          (card_),
+	m_syntax_error  (false),
+	m_keep_open     (false),
+
 	m_cancel_button (NULL),
 	m_add_n_close_button (NULL),
 	m_add_button    (NULL),
 	m_help_button   (NULL),
+
 	m_front_text    (NULL),
 	m_back_text     (NULL),
 	m_example_text  (NULL),
+	m_altspell_text (NULL),
+
 	m_edit_controls (NULL),
-	m_syntax_error  (false),
-	m_keep_open     (false),
 	m_help_dialog   (*this)
 {
 	trace_with_mask("CardView::CardView(VCard&)",GUITRACE);
@@ -80,36 +111,40 @@ CardView (VCard* card_) :
 	Gtk::Alignment* alignment6;
 	Gtk::Alignment* alignment7;
 	Gtk::Alignment* alignment8;
+	Gtk::Alignment* alignment9;
 
 	Gtk::ScrolledWindow* front_scrollwin;
 	Gtk::ScrolledWindow* backtext_scrollwin;
 	Gtk::ScrolledWindow* example_scrollwin;
+	Gtk::ScrolledWindow* altspell_scrollwin;
 
 	Gtk::Label* back_label;
 	Gtk::Label* example_label;
 	Gtk::Label* front_label;
+	Gtk::Label* altspell_label;
 
 	Gtk::VBox* vbox_frontlabel;
 	Gtk::VBox* vbox_backlabel;
 	Gtk::VBox* vbox_examplelabel;
+	Gtk::VBox* vbox_altspelllabel;
 
 	Gtk::Table* elems_table;
 	Gtk::VBox*  vbox_all;
 
-#ifdef IS_HILDON
+#if defined (IS_HILDON)
 	set_size_request (672, 396);
 #else
-	set_icon_from_file (GRAPPDATADIR "/pixmaps/cardview_32x32.png");
+	set_icon_from_file (GRAPPDATDIR "/pixmaps/cardview_32x32.png");
 
 	Gdk::Rectangle geom = CONFIG->get_card_view_geometry ();
 	set_size_request (geom.get_width (), geom.get_height ());
 
 	set_title ("Card View");
 	set_resizable (true);
-	set_modal (true);
+	set_modal (true);  // very important - we're not protected from multi-clicks
 	set_transient_for (*MAINWIN);
 	property_window_position ().set_value (Gtk::WIN_POS_CENTER);
-#endif
+#endif	// ifdef IS_HILDON
 
 	/** <Clear> button
 	 */
@@ -159,13 +194,13 @@ CardView (VCard* card_) :
 	 */
 	m_front_text       = Gtk::manage (new Gtk::TextView);
 	front_scrollwin    = Gtk::manage (new Gtk::ScrolledWindow);
-	front_label        = Gtk::manage (new Gtk::Label (_("Front : ")));
+	front_label        = Gtk::manage (new Gtk::Label (_("Front :\n \n \n")));
 	alignment6         = Gtk::manage (new Gtk::Alignment (0.84, 0.5, 0, 0));
 	vbox_frontlabel    = Gtk::manage (new Gtk::VBox (false, 0));
 
 	m_back_text        = Gtk::manage (new Gtk::TextView);
 	backtext_scrollwin = Gtk::manage (new Gtk::ScrolledWindow);
-	back_label         = Gtk::manage (new Gtk::Label (_("Back :")));
+	back_label         = Gtk::manage (new Gtk::Label (_("Back :\n \n \n")));
 	alignment5         = Gtk::manage (new Gtk::Alignment (0.76, 0.5, 0, 0));
 	vbox_backlabel     = Gtk::manage (new Gtk::VBox (false, 0));
 
@@ -175,21 +210,23 @@ CardView (VCard* card_) :
 	alignment7         = Gtk::manage (new Gtk::Alignment (0.84, 0.5, 0, 0));
 	vbox_examplelabel  = Gtk::manage (new Gtk::VBox (false, 0));
 
-	elems_table = Gtk::manage (new Gtk::Table (2, 2, false));
+	m_altspell_text     = Gtk::manage (new Gtk::TextView);
+	altspell_scrollwin  = Gtk::manage (new Gtk::ScrolledWindow);
+	altspell_label      = Gtk::manage (new Gtk::Label (_("Alt\nSpelling : ")));
+	alignment9          = Gtk::manage (new Gtk::Alignment (0.84, 0.5, 0, 0));
+	vbox_altspelllabel  = Gtk::manage (new Gtk::VBox (false, 0));
+
+	elems_table = Gtk::manage (new Gtk::Table (2, 5, false));
 	vbox_all = Gtk::manage (new Gtk::VBox (false, 0));
 
-	m_front_text  ->modify_font (CONFIG->get_example_font  ());
-	m_back_text   ->modify_font (CONFIG->get_example_font  ());
-	m_example_text->modify_font (CONFIG->get_example_font  ());
+	m_front_text   ->modify_font (CONFIG->get_example_font ());
+	m_back_text    ->modify_font (CONFIG->get_example_font ());
+	m_example_text ->modify_font (CONFIG->get_example_font ());
+	m_altspell_text->modify_font (CONFIG->get_example_font ());
 
 	image1->set_alignment  (0.5,0.5);
 	image1->set_padding    (0,0);
-	label1->set_alignment  (0.5,0.5);
-	label1->set_padding    (0,0);
-	label1->set_justify    (Gtk::JUSTIFY_LEFT);
-	label1->set_line_wrap  (false);
-	label1->set_use_markup (false);
-	label1->set_selectable (false);
+	SET_CV_LABEL(label1);
 
 	hbox1->pack_start (*image1, Gtk::PACK_SHRINK, 0);
 	hbox1->pack_start (*label1, Gtk::PACK_SHRINK, 0);
@@ -197,12 +234,7 @@ CardView (VCard* card_) :
 
 	image2->set_alignment  (0.5,0.5);
 	image2->set_padding    (0,0);
-	label2->set_alignment  (0.5,0.5);
-	label2->set_padding    (0,0);
-	label2->set_justify    (Gtk::JUSTIFY_LEFT);
-	label2->set_line_wrap  (false);
-	label2->set_use_markup (false);
-	label2->set_selectable (false);
+	SET_CV_LABEL(label2);
 	hbox2->pack_start (*image2, Gtk::PACK_SHRINK, 0);
 	hbox2->pack_start (*label2, Gtk::PACK_SHRINK, 0);
 	alignment2->add (*hbox2);
@@ -213,12 +245,7 @@ CardView (VCard* card_) :
 
 	image3->set_alignment  (0.5,0.5);
 	image3->set_padding    (0,0);
-	label3->set_alignment  (0.5,0.5);
-	label3->set_padding    (0,0);
-	label3->set_justify    (Gtk::JUSTIFY_LEFT);
-	label3->set_line_wrap  (false);
-	label3->set_use_markup (false);
-	label3->set_selectable (false);
+	SET_CV_LABEL(label3);
 	hbox3->pack_start (*image3, Gtk::PACK_SHRINK, 0);
 	hbox3->pack_start (*label3, Gtk::PACK_SHRINK, 0);
 	alignment3->add (*hbox3);
@@ -229,12 +256,7 @@ CardView (VCard* card_) :
 
 	image4->set_alignment  (0.5,0.5);
 	image4->set_padding    (0,0);
-	label4->set_alignment  (0.5,0.5);
-	label4->set_padding    (0,0);
-	label4->set_justify    (Gtk::JUSTIFY_LEFT);
-	label4->set_line_wrap  (false);
-	label4->set_use_markup (false);
-	label4->set_selectable (false);
+	SET_CV_LABEL(label4);
 	hbox4->pack_start (*image4, Gtk::PACK_SHRINK, 0);
 	hbox4->pack_start (*label4, Gtk::PACK_SHRINK, 0);
 	alignment4->add (*hbox4);
@@ -245,12 +267,7 @@ CardView (VCard* card_) :
 
 	image5->set_alignment  (0.5,0.5);
 	image5->set_padding    (0,0);
-	label5->set_alignment  (0.5,0.5);
-	label5->set_padding    (0,0);
-	label5->set_justify    (Gtk::JUSTIFY_LEFT);
-	label5->set_line_wrap  (false);
-	label5->set_use_markup (false);
-	label5->set_selectable (false);
+	SET_CV_LABEL(label5);
 	hbox5->pack_start (*image5, Gtk::PACK_SHRINK, 0);
 	hbox5->pack_start (*label5, Gtk::PACK_SHRINK, 0);
 	alignment8->add (*hbox5);
@@ -259,20 +276,30 @@ CardView (VCard* card_) :
 	m_help_button->set_relief (Gtk::RELIEF_NORMAL);
 	m_help_button->add (*alignment8);
 
-	// Back Text
-	m_back_text->set_flags (Gtk::CAN_FOCUS);
-	m_back_text->set_border_width (3);
-	m_back_text->set_editable (true);
-	m_back_text->set_cursor_visible (true);
-	m_back_text->set_pixels_above_lines (3);
-	m_back_text->set_pixels_below_lines (0);
-	m_back_text->set_pixels_inside_wrap (0);
-	m_back_text->set_left_margin  (4);
-	m_back_text->set_right_margin (4);
-	m_back_text->set_indent (0);
-	m_back_text->set_wrap_mode (Gtk::WRAP_WORD);
-	m_back_text->set_justification (Gtk::JUSTIFY_LEFT);
-	m_back_text->set_accepts_tab (false);
+	/**--------------**
+	 **  Front Text  **
+	 **--------------**
+	 */
+	SET_CV_TEXT(m_front_text);
+	m_front_text->get_buffer ()->set_text (_("Fill up the front of the card"));
+
+	front_scrollwin->set_flags (Gtk::CAN_FOCUS);
+	front_scrollwin->set_shadow_type (Gtk::SHADOW_NONE);
+	front_scrollwin->set_policy (Gtk::POLICY_ALWAYS, Gtk::POLICY_ALWAYS);
+	front_scrollwin->
+		property_window_placement ().set_value (Gtk::CORNER_TOP_LEFT);
+	front_scrollwin->add (*m_front_text);
+
+	SET_CV_LABEL(front_label);
+	alignment6->add (*front_label);
+	vbox_frontlabel->pack_start (*alignment6, Gtk::PACK_SHRINK, 0);
+	
+
+	/**--------------**
+	 **   Back Text  **
+	 **--------------**
+	 */
+	SET_CV_TEXT(m_back_text);
 	m_back_text->get_buffer ()->set_text (_("Fill up the back of the card"));
 
 	backtext_scrollwin->set_flags (Gtk::CAN_FOCUS);
@@ -282,29 +309,15 @@ CardView (VCard* card_) :
 		property_window_placement ().set_value (Gtk::CORNER_TOP_LEFT);
 	backtext_scrollwin->add (*m_back_text);
 
-	back_label->set_alignment (0.5,0.5);
-	back_label->set_padding (0,0);
-	back_label->set_justify (Gtk::JUSTIFY_LEFT);
-	back_label->set_line_wrap (false);
-	back_label->set_use_markup (false);
-	back_label->set_selectable (false);
+	SET_CV_LABEL(back_label);
 	alignment5->add (*back_label);
 	vbox_backlabel->pack_start (*alignment5, Gtk::PACK_SHRINK, 0);
 
-	// Example Text
-	m_example_text->set_flags (Gtk::CAN_FOCUS);
-	m_example_text->set_border_width (3);
-	m_example_text->set_editable (true);
-	m_example_text->set_cursor_visible (true);
-	m_example_text->set_pixels_above_lines (3);
-	m_example_text->set_pixels_below_lines (0);
-	m_example_text->set_pixels_inside_wrap (0);
-	m_example_text->set_left_margin  (4);
-	m_example_text->set_right_margin (4);
-	m_example_text->set_indent (0);
-	m_example_text->set_wrap_mode (Gtk::WRAP_WORD);
-	m_example_text->set_justification (Gtk::JUSTIFY_LEFT);
-	m_example_text->set_accepts_tab (false);
+	/**----------------**
+	 **  Example Text  **
+	 **----------------**
+	 */
+	SET_CV_TEXT(m_example_text);
 	m_example_text->get_buffer ()->set_text (_("Fill up an example"));
 
 	example_scrollwin->set_flags (Gtk::CAN_FOCUS);
@@ -314,15 +327,29 @@ CardView (VCard* card_) :
 		property_window_placement ().set_value (Gtk::CORNER_TOP_LEFT);
 	example_scrollwin->add (*m_example_text);
 
-	example_label->set_alignment (0.5,0.5);
-	example_label->set_padding (0,0);
-	example_label->set_justify (Gtk::JUSTIFY_RIGHT);
-	example_label->set_line_wrap (false);
-	example_label->set_use_markup (false);
-	example_label->set_selectable (false);
+	SET_CV_LABEL(example_label);
 	vbox_examplelabel->pack_start (*example_label, Gtk::PACK_SHRINK, 0);
 
-	/** Add EditControls
+	/**-----------------**
+	 **  AltSpell Text  **
+	 **-----------------**
+	 */
+	SET_CV_TEXT(m_altspell_text);
+
+	altspell_scrollwin->set_flags (Gtk::CAN_FOCUS);
+	altspell_scrollwin->set_shadow_type (Gtk::SHADOW_NONE);
+	altspell_scrollwin->set_policy (Gtk::POLICY_ALWAYS, Gtk::POLICY_ALWAYS);
+	altspell_scrollwin->
+		property_window_placement ().set_value (Gtk::CORNER_TOP_LEFT);
+	altspell_scrollwin->add (*m_altspell_text);
+
+	SET_CV_LABEL(altspell_label);
+	alignment9->add (*altspell_label);
+	vbox_altspelllabel->pack_start (*alignment9, Gtk::PACK_SHRINK, 0);
+
+	/**----------------**
+	 **  Edit Controls **
+	 **----------------**
 	 */
 	m_edit_controls = Gtk::manage(new EditControls(m_front_text->get_buffer()));
 	vbox_examplelabel->pack_end (*m_edit_controls, Gtk::PACK_SHRINK, 0);
@@ -342,6 +369,11 @@ CardView (VCard* card_) :
 									   &EditControls::on_focus_changed),
 							  m_example_text));
 
+	m_altspell_text->signal_grab_focus ().connect (
+		bind<Gtk::TextView*> (mem_fun (*m_edit_controls, 
+									   &EditControls::on_focus_changed),
+							  m_altspell_text));
+
 #ifdef IS_HILDON
 	/** Trap 'on virtual keyboard close' event to restore
 	 *  original size of the dialog
@@ -350,39 +382,6 @@ CardView (VCard* card_) :
 		mem_fun (*this, &CardView::hildon_im_close_cb));
 #endif
 
-	/** Front Text
-	 */
-	m_front_text->set_flags (Gtk::CAN_FOCUS);
-	m_front_text->set_border_width (3);
-	m_front_text->set_editable (true);
-	m_front_text->set_cursor_visible (true);
-	m_front_text->set_pixels_above_lines (3);
-	m_front_text->set_pixels_below_lines (0);
-	m_front_text->set_pixels_inside_wrap (0);
-	m_front_text->set_left_margin  (4);
-	m_front_text->set_right_margin (4);
-	m_front_text->set_indent (0);
-	m_front_text->set_wrap_mode (Gtk::WRAP_WORD);
-	m_front_text->set_justification (Gtk::JUSTIFY_LEFT);
-	m_front_text->set_accepts_tab (false);
-	m_front_text->get_buffer ()->set_text (_("Fill up the front of the card"));
-
-	front_scrollwin->set_flags (Gtk::CAN_FOCUS);
-	front_scrollwin->set_shadow_type (Gtk::SHADOW_NONE);
-	front_scrollwin->set_policy (Gtk::POLICY_ALWAYS, Gtk::POLICY_ALWAYS);
-	front_scrollwin->
-		property_window_placement ().set_value (Gtk::CORNER_TOP_LEFT);
-	front_scrollwin->add (*m_front_text);
-
-	front_label->set_alignment (0.5,0.5);
-	front_label->set_padding (0,0);
-	front_label->set_justify (Gtk::JUSTIFY_LEFT);
-	front_label->set_line_wrap (false);
-	front_label->set_use_markup (false);
-	front_label->set_selectable (false);
-	alignment6->add (*front_label);
-	vbox_frontlabel->pack_start (*alignment6, Gtk::PACK_SHRINK, 0);
-	
 	/**
 	 *  Table holds elements together:
 	 *
@@ -394,26 +393,37 @@ CardView (VCard* card_) :
 	elems_table->set_row_spacings (8);
 	elems_table->set_col_spacings (2);
 	elems_table->set_name ("CardViewElemsTable");
-
+	
+	/*-----------------------------------------------------------*/
 	elems_table->attach (*vbox_frontlabel, 0, 1, 0, 1, 
 						Gtk::FILL, Gtk::FILL, 0, 0);
 
 	elems_table->attach (*front_scrollwin, 1, 2, 0, 1, 
 						Gtk::EXPAND|Gtk::FILL, Gtk::FILL, 0, 0);
 
+	/*-----------------------------------------------------------*/
 	elems_table->attach (*vbox_backlabel, 0, 1, 1, 2, 
 						Gtk::FILL, Gtk::FILL, 0, 0);
 
 	elems_table->attach (*backtext_scrollwin, 1, 2, 1, 2, 
 						Gtk::EXPAND|Gtk::FILL, Gtk::FILL, 0, 0);
 
+	/*-----------------------------------------------------------*/
 	elems_table->attach (*vbox_examplelabel, 0, 1, 2, 3, 
 						Gtk::FILL, Gtk::EXPAND|Gtk::FILL, 0, 0);
 
 	elems_table->attach (*example_scrollwin, 1, 2, 2, 3, 
 						Gtk::FILL, Gtk::FILL, 0, 0);
 
-	elems_table->attach (*alignment7, 0, 1, 3, 4, 
+	/*-----------------------------------------------------------*/
+	elems_table->attach (*vbox_altspelllabel, 0, 1, 3, 4, 
+						Gtk::FILL, Gtk::EXPAND|Gtk::FILL, 0, 0);
+
+	elems_table->attach (*altspell_scrollwin, 1, 2, 3, 4, 
+						Gtk::FILL, Gtk::FILL, 0, 0);
+
+	/*-----------------------------------------------------------*/
+	elems_table->attach (*alignment7, 0, 1, 4, 5, 
 						 Gtk::FILL, Gtk::FILL, 0, 0);
 
 #ifdef IS_HILDON
@@ -432,7 +442,9 @@ CardView (VCard* card_) :
 	vbox_all->pack_start (*elems_table);
 #endif
 
-	/** Create and pack buttons
+	/**-------------------------**
+	 ** Create and pack buttons **
+	 **-------------------------**
 	 */
 	Gtk::HBox* hbbox = Gtk::manage(new Gtk::HBox(false, 0));
 	Gtk::HSeparator* hseparator = manage (new Gtk::HSeparator);
@@ -454,7 +466,9 @@ CardView (VCard* card_) :
 
 	add (*vbox_all);
 
-	/** Setup signals
+	/**---------------**
+	 ** Setup signals **
+	 **---------------**
 	 */
 	m_help_button->signal_clicked ().connect (
 		sigc::mem_fun (*this, &CardView::on_help_clicked));
@@ -471,12 +485,16 @@ CardView (VCard* card_) :
     signal_size_allocate ().connect (
 		sigc::mem_fun (*this, &CardView::size_allocate_cb));
 
-	/** Fill up the form
+	/**------------------**
+	 ** Fill up the form **
+	 **------------------**
 	 */
-	if (m_card != NULL) {
-		m_front_text  ->get_buffer ()->set_text (m_card->get_question ());
-		m_back_text   ->get_buffer ()->set_text (m_card->get_answer   ());
-		m_example_text->get_buffer ()->set_text (m_card->get_example  ());
+	if (m_card != NULL) 
+	{
+		m_front_text   ->get_buffer ()->set_text (m_card->get_question ());
+		m_back_text    ->get_buffer ()->set_text (m_card->get_answer   ());
+		m_example_text ->get_buffer ()->set_text (m_card->get_example  ());
+		m_altspell_text->get_buffer ()->set_text (m_card->get_alt_spelling  ());
 	}
 
 	/** Set Widget names - very helpful with debugging!
@@ -484,6 +502,7 @@ CardView (VCard* card_) :
 	m_front_text->set_name    ("FrontText");
 	m_back_text ->set_name    ("BackText");
 	m_example_text->set_name  ("ExampleText");
+	m_altspell_text->set_name ("AltSpellText");
 	m_help_button->set_name   ("HelpButton");
 	m_cancel_button->set_name ("CancelButton");
 	m_add_button->set_name    ("AddButton");
@@ -547,27 +566,29 @@ run (DeckView& deckview_)
 {
 	trace_with_mask("CardView::run(DeckView&)",GUITRACE);
 
-	int ret = 0;
 	m_syntax_error = false;
 	take_snapshot ();
 
 #ifdef IS_HILDON
 	GRANULE->register_appview (this);
 	Gtk::Main::run (*HILDONAPPWIN);
+
 #else
-	set_transient_for (deckview_);
 	Gtk::Main::run (*this);
 #endif
 
 	DL ((GRAPP,"Processing syntax_error and response...\n"));
-	if (m_syntax_error) {
-		DL ((GRAPP,"Caught a syntax error (RESPONSE_REJECT)\n"));
+
+	if (m_syntax_error) 
+	{
+		DL ((GRAPP,"Caught syntax error (RESPONSE_REJECT)\n"));
 		delete m_card;
 		m_response = Gtk::RESPONSE_REJECT;
 		goto done;
 	}
 
-	if (m_response == Gtk::RESPONSE_CANCEL) {
+	if (m_response == Gtk::RESPONSE_CANCEL) 
+	{
 		DL ((GRAPP,"Dialog cancelled (RESPONSE_CANCEL)\n"));
 		delete m_card;
 		goto done;
@@ -688,10 +709,12 @@ on_addnclose_clicked  ()
 
 	/** Transfer data from the widget to the card
 	 */
-	if (m_card != NULL && text_modified ()) {
-		m_card->set_question (m_front_text  ->get_buffer ()->get_text ());
-		m_card->set_answer   (m_back_text   ->get_buffer ()->get_text ());
-		m_card->set_example  (m_example_text->get_buffer ()->get_text ());
+	if (m_card != NULL && text_modified ()) 
+	{
+		m_card->set_question     (m_front_text   ->get_buffer ()->get_text ());
+		m_card->set_answer       (m_back_text    ->get_buffer ()->get_text ());
+		m_card->set_example      (m_example_text ->get_buffer ()->get_text ());
+		m_card->set_alt_spelling (m_altspell_text->get_buffer ()->get_text ());
 	}
 	m_keep_open = false;
 	m_front_text->grab_focus ();
@@ -712,18 +735,20 @@ take_snapshot ()
 {
 	trace_with_mask("CardView::take_snapshot",GUITRACE);
 
-	m_snapshot_front   = m_front_text  ->get_buffer ()->get_text ();
-	m_snapshot_back    = m_back_text   ->get_buffer ()->get_text ();
-	m_snapshot_example = m_example_text->get_buffer ()->get_text ();
+	m_snapshot_front    = m_front_text   ->get_buffer ()->get_text ();
+	m_snapshot_back     = m_back_text    ->get_buffer ()->get_text ();
+	m_snapshot_example  = m_example_text ->get_buffer ()->get_text ();
+	m_snapshot_altspell = m_altspell_text->get_buffer ()->get_text ();
 }
 
 bool
 CardView::
 text_modified ()
 {
-	return (m_snapshot_front   != m_front_text  ->get_buffer ()->get_text () ||
-			m_snapshot_back    != m_back_text   ->get_buffer ()->get_text () ||
-			m_snapshot_example != m_example_text->get_buffer ()->get_text ());
+	return (m_snapshot_front   != m_front_text   ->get_buffer ()->get_text () ||
+			m_snapshot_back    != m_back_text    ->get_buffer ()->get_text () ||
+			m_snapshot_example != m_example_text ->get_buffer ()->get_text () ||
+			m_snapshot_altspell != m_altspell_text->get_buffer ()->get_text ());
 }
 
 void
@@ -754,10 +779,12 @@ clear_dialog ()
 	m_front_text  ->get_buffer ()->set_text ("");
 	m_back_text   ->get_buffer ()->set_text ("");
 	m_example_text->get_buffer ()->set_text ("");
+	m_altspell_text->get_buffer ()->set_text ("");
 
-	m_snapshot_front   = "";
-	m_snapshot_back    = "";
-	m_snapshot_example = "";
+	m_snapshot_front    = "";
+	m_snapshot_back     = "";
+	m_snapshot_example  = "";
+	m_snapshot_altspell = "";
 }
 
 void
