@@ -2,7 +2,7 @@
 //------------------------------------------------------------------------------
 //                              CardView.h
 //------------------------------------------------------------------------------
-// $Id: CardView.h,v 1.19 2006/03/19 15:39:44 vlg Exp $
+// $Id: CardView.h,v 1.20 2006/09/06 02:07:59 vlg Exp $
 //
 //  Copyright (c) 2004-2005 by Vladislav Grinchenko
 //
@@ -97,11 +97,14 @@ private:
 	Gtk::TextView*  m_front_text;
 	Gtk::TextView*  m_back_text;
 	Gtk::TextView*  m_example_text;
+	Gtk::TextView*  m_altspell_text;
 
 	// -*- Snapshot to detect true changes -*-
+	//
 	Glib::ustring   m_snapshot_front;
 	Glib::ustring   m_snapshot_back;
 	Glib::ustring   m_snapshot_example;
+	Glib::ustring   m_snapshot_altspell;
 
 	EditControls*   m_edit_controls;
 
