@@ -2,7 +2,7 @@
 //------------------------------------------------------------------------------
 //                              Deck.cpp
 //------------------------------------------------------------------------------
-// $Id: Deck.cpp,v 1.44 2006/03/06 03:59:38 vlg Exp $
+// $Id: Deck.cpp,v 1.49 2006/09/30 20:42:27 vlg Exp $
 //
 //  Copyright (c) 2004-2006 by Vladislav Grinchenko
 //
@@ -18,8 +18,10 @@
 using std::ofstream;
 using std::ifstream;
 
+#include <glib.h>
+#include <glib/gstdio.h>
+
 #ifdef IS_HILDON
-#  include <glib.h>
 #  include <hildon-widgets/hildon-system-sound.h>
 #endif
 
@@ -33,7 +35,7 @@ using std::ifstream;
 #include "Deck.h"
 #include "Card.h"
 #include "MainWindow.h"
-#include "Config.h"
+#include "GrappConf.h"
 #include "CSVImportDialog.h"
 #include "Intern.h"
 
@@ -133,10 +135,14 @@ load (const string& fname_, DeckAction action_)
 		emsg.run ();
 		return -1;
 	}
+	/** Setting side selection should come before callback to DECKMGR
+	 */
+	set_side_selection (CONFIG->get_flipside_history (get_name ()));
 
 	if (action_ == INSTALL_DECK) {
 		DECKMGR->on_deck_loaded (this, LOAD_OK);
 	}
+
 	m_status = CLEAN_DECK;
 	dump_deck_status ();
 
@@ -215,6 +221,13 @@ parse_xml_tree (xmlDocPtr parser_, std::string& error_msg_)
 		card->set_question ((const char*) result);
 		xmlFree (result);
 
+		sprintf (xpath, "/deck/card[%d]/front/@alt_spelling", cards_counter);
+		result = Granule::get_node_by_xpath (parser_, xpath);
+		if (result != NULL) {
+			card->set_alt_spelling ((const char*) result);
+			xmlFree (result);
+		}
+
 		sprintf (xpath, "/deck/card[%d]/back", cards_counter);
 		result = Granule::get_node_by_xpath (parser_, xpath);
 		if (result != NULL) {	                              /* can be empty */
@@ -417,8 +430,8 @@ save_as (const string& fname_)
 		Gtk::MessageDialog qm (msg, false,
 							   Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_NONE,
 							   true);
-		qm.add_button ("Yes", Gtk::RESPONSE_YES);
-		qm.add_button ("No",  Gtk::RESPONSE_NO);
+		qm.add_button (Gtk::Stock::NO,  Gtk::RESPONSE_NO);
+		qm.add_button (Gtk::Stock::YES, Gtk::RESPONSE_YES);
 		if (qm.run () == Gtk::RESPONSE_NO) {
 			return -1;
 		}
@@ -448,11 +461,12 @@ save ()
 	static const char dtd_url [] = "http://granule.sourceforge.net/granule.dtd";
 	int ret = 0;
 	xmlTextWriterPtr writer;
-
-	::unlink (m_fname.c_str ());
+	Glib::ustring alt_spell;
+	g_remove (m_fname.c_str ());
 
 	writer = xmlNewTextWriterFilename (m_fname.c_str (), 0);
 	if (writer == NULL) {
+		DL ((GRAPP,"Failed to save Deck \"%s\"\n", m_fname.c_str ()));
 		return -1;
 	}
 
@@ -483,16 +497,29 @@ save ()
 						 BAD_CAST Glib::locale_to_utf8 (m_snd_path).c_str ());
 
 	cardlist_const_iterator citer = m_cards.begin ();
-	while (citer != m_cards.end ()) {
+	while (citer != m_cards.end ()) 
+	{
 		ret = xmlTextWriterStartElement (writer, BAD_CAST "card");
 		ret = xmlTextWriterWriteAttribute (writer, BAD_CAST "id",
 							   BAD_CAST (*citer)->get_id_str ().c_str ());
-		ret = xmlTextWriterWriteElement (writer, BAD_CAST "front",
+
+		ret = xmlTextWriterStartElement (writer, BAD_CAST "front");
+
+		alt_spell = (*citer)->get_alt_spelling ();
+		if (alt_spell.length () > 0) {
+			ret = xmlTextWriterWriteAttribute (writer, BAD_CAST "alt_spelling",
+											   BAD_CAST alt_spell.c_str ());
+		}
+		ret = xmlTextWriterWriteString (writer,
 								 BAD_CAST (*citer)->get_question ().c_str ());
+		ret = xmlTextWriterEndElement (writer);	// @end front
+
 		ret = xmlTextWriterWriteElement (writer, BAD_CAST "back",
 								 BAD_CAST (*citer)->get_answer ().c_str ());
+
 		ret = xmlTextWriterWriteElement (writer, BAD_CAST "back_example",
 								 BAD_CAST (*citer)->get_example ().c_str ());
+
 		ret = xmlTextWriterEndElement (writer);	// @end card
 		citer++;
 	}
@@ -575,13 +602,18 @@ play_card (Deck::cardlist_iterator& iter_)
 		return;
 	}
 	std::vector<string> v;
-	string w = Glib::locale_from_utf8 ((*iter_)->get_question ());
-	ASSA::Utils::split (w.c_str (), v);
-	if (v.size () > 1) {
-		if (v[0] != "to") {
-			return;
+	string w;
+
+	w = Glib::locale_from_utf8 ((*iter_)->get_alt_spelling ());
+	if (w.length () == 0) {
+		w = Glib::locale_from_utf8 ((*iter_)->get_question ());
+		ASSA::Utils::split (w.c_str (), v);
+		if (v.size () > 1) {
+			if (v[0] != "to") {
+				return;
+			}
+			w = v[1];
 		}
-		w = v[1];
 	}
 	string sfn (m_snd_path + G_DIR_SEPARATOR_S);
 	sfn += w[0];
@@ -593,10 +625,17 @@ play_card (Deck::cardlist_iterator& iter_)
 		return;
 	}
 
-#ifdef IS_HILDON
-	hildon_play_system_sound (sfn.c_str ());
 	DL((APP,"Playing \"%s\" WAV\n", sfn.c_str ()));
-#else
+
+#if defined (WIN32)
+
+	PlaySound (sfn.c_str (), NULL, SND_FILENAME| SND_ASYNC);
+
+#elif defined (IS_HILDON)
+
+	hildon_play_system_sound (sfn.c_str ());
+
+#else  /* POSIX */
 
 	snprintf (execmdbuf, 512, CONFIG->get_snd_player_args (), sfn.c_str ());
 	DL((APP,"Playing \"%s %s\"\n", CONFIG->get_snd_player_cmd (), execmdbuf));
