@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //------------------------------------------------------------------------------
-// $Id: DeckList.cpp,v 1.16 2006/03/19 15:39:44 vlg Exp $
+// $Id: DeckList.cpp,v 1.18 2006/09/21 01:25:20 vlg Exp $
 //------------------------------------------------------------------------------
 //                            DeckList.cpp
 //------------------------------------------------------------------------------
@@ -243,7 +243,6 @@ on_drag_begin (const Glib::RefPtr<Gdk::DragContext>& context_)
 	int cell_width;
 	int cell_height;
 	int x_pos; 
-	int y_pos;
 	Gdk::Rectangle area;
 
 	Gtk::TreeModel::iterator iter = m_tree_sel_ref->get_selected ();
@@ -252,10 +251,6 @@ on_drag_begin (const Glib::RefPtr<Gdk::DragContext>& context_)
 		 */
 		m_tree_view.get_column (0)->cell_get_size (area, x_off, y_off,
 												   cell_width, cell_height);
-
-		/** Get the cursor position for horizontal displacement.
-		m_tree_view.get_pointer (x_pos, y_pos);
-		*/
 		if (cell_height < 0) { 
 			cell_height = 0;	
 		}
@@ -264,7 +259,7 @@ on_drag_begin (const Glib::RefPtr<Gdk::DragContext>& context_)
 		x_pos = 20;
 
 		Glib::RefPtr<Gdk::Pixbuf> dnd_pixbuf =
-		   Gdk::Pixbuf::create_from_file(GRAPPDATADIR "/pixmaps/copy_deck.xpm");
+		   Gdk::Pixbuf::create_from_file(GRAPPDATDIR "/pixmaps/copy_deck.png");
 
 		context_->set_icon (dnd_pixbuf, x_pos, cell_height/2);
 	}
