@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //------------------------------------------------------------------------------
-// $Id: DeckPlayer.cpp,v 1.90 2006/03/19 15:39:44 vlg Exp $
+// $Id: DeckPlayer.cpp,v 1.102 2006/10/08 02:46:30 vlg Exp $
 //------------------------------------------------------------------------------
 //                            DeckPlayer.cpp
 //------------------------------------------------------------------------------
@@ -46,10 +46,6 @@ using sigc::mem_fun;
 #include "CardView.h"
 #include "Intern.h"
 
-//------------------------------------------------------------------------------
-// VerifyControl functions
-//------------------------------------------------------------------------------
-
 static const Glib::ustring clabels [] = { 
 	N_("Check"), N_("Next "), N_("Correct  "), N_("Incorrect"), 
 	"               "
@@ -57,18 +53,22 @@ static const Glib::ustring clabels [] = {
 
 enum { CHECK_ANSWER, NEXT, CORRECT, INCORRECT, EMPTY };
 
-static const gchar* prefixes [] = { "to ", "a ", "an ", NULL };
 static const gchar* states [] = { "VC_START", "VC_ARMED", "VC_VALIDATED" };
 
+/*******************************************************************************
+ *                       VerifyControl Member Functions                        *
+ *******************************************************************************
+ */
 VerifyControl::
-VerifyControl (DeckPlayer& parent_) : 
+VerifyControl (DeckPlayer& parent_) 
+	: 
 	Gtk::Table  (2, 2, false),
 	m_parent    (parent_),
+	m_state     (VC_START),
 	m_red       ("red"),
 	m_green     ("MediumSeaGreen"),
 	m_white     ("white"),
 	m_black     ("black"),
-	m_state     (VC_START),
 	m_test_line (NULL),
 	m_line_num  (1)
 	
@@ -151,7 +151,7 @@ void
 VerifyControl::
 set_focus_on_entry () 
 { 
-    trace_with_mask("VerifyControl::set_focus_on_entry", GUITRACE);
+    trace_with_mask("VerifyControl::set_focus_on_entry", KEYIN);
 	m_entry->grab_focus (); 
 }
 
@@ -171,7 +171,6 @@ on_test_line_changed ()
  * Here is the tricky part. The answer_ can be marked up with
  * Pango markup - so strip it before comparing. Strip all well-known
  * prefixes as well and reduce multiple white spaces to a single one.
- * Also, at the moment, we just compare the first line of the answer. 
  */
 void
 VerifyControl::
@@ -193,7 +192,8 @@ compare (const ustring& answer_)
 		the m_line_num > 1, then we fetch that line instead.
 		If we are short of lines, that's a failed match.
 	*/
-	if (m_parent.get_selected_size () == BACK && m_line_num > 1) {
+	if (m_parent.get_selected_size () == BACK && m_line_num > 1) 
+	{
 		std::vector<Glib::ustring> vec;
 		std::istringstream input (tmp.c_str ());
 		Glib::ustring token;
@@ -239,8 +239,11 @@ compare (const ustring& answer_)
 		*/
 		Glib::ustring lhs (text1);
 		Glib::ustring rhs (text2);
-		for (int idx = 0; idx < lhs.size () && idx < rhs.size (); idx++) {
-			if (lhs [idx] != rhs [idx]) {
+
+		for (int idx = 0; idx < lhs.size () && idx < rhs.size (); idx++) 
+		{
+			if (lhs [idx] != rhs [idx]) 
+			{
 				m_entry->select_region (idx, -1);
 				break;
 			}
@@ -258,7 +261,7 @@ void
 VerifyControl::
 on_entry_changed ()
 {
-    trace_with_mask("VerifyControl::on_entry_changed", GUITRACE);
+    trace_with_mask("VerifyControl::on_entry_changed", KEYIN);
 
 	m_button->set_sensitive (true);
 
@@ -272,7 +275,7 @@ on_entry_changed ()
 		str += m_entry->get_text ();
 		str += "</span>";
 		m_entry->get_layout ()->set_markup (str);
-		DL ((GRAPP,"Gtk::Entry markup enabled\n"));
+		DL ((KEYIN,"Gtk::Entry markup enabled\n"));
 	}
 }
 
@@ -314,32 +317,37 @@ set_state (VCState state_)
 	}
 }
 
-//------------------------------------------------------------------------------
-// DeckPlayer functions
-//------------------------------------------------------------------------------
-Gtk::StockID DeckPlayer::SAY_IT ("say-it");
-Gtk::StockID DeckPlayer::THUMB_UP ("thumb-up");
+/*******************************************************************************
+ *                         DeckPlayer Member Functions                         *
+ *******************************************************************************
+ */
+Gtk::StockID DeckPlayer::SAY_IT     ("say-it"    );
+Gtk::StockID DeckPlayer::THUMB_UP   ("thumb-up"  );
 Gtk::StockID DeckPlayer::THUMB_DOWN ("thumb-down");
 
 DeckPlayer::
 DeckPlayer (VDeck& vdeck_) 
 		:
+
 #ifdef IS_HILDON
 	Hildon::AppView ("Deck Player"),
 	m_hildon_fullscreen (false),
 #endif
-    m_close_button      (
-		manage (new ButtonWithImageLabel (Gtk::Stock::CLOSE, _("Close")))),
-    m_shuffle_button    (
-		manage (new ButtonWithImageLabel (Gtk::Stock::REFRESH, _("Shuffle")))),
-    m_edit_button       (
-      manage (new ButtonWithImageLabel (Gtk::Stock::DND_MULTIPLE, 
-										_("View/Edit Deck")))),
-    m_vcard_edit_button (
-		manage (new ButtonWithImageLabel (Gtk::Stock::DND, _("Edit Card")))),
+
+    m_close_button      (manage (new ButtonWithImageLabel (
+									 Gtk::Stock::CLOSE, _("Close")))),
+    m_shuffle_button    (manage (new ButtonWithImageLabel (
+									 Gtk::Stock::REFRESH, _("Shuffle")))),
+    m_edit_button       (manage (new ButtonWithImageLabel (
+									 Gtk::Stock::DND_MULTIPLE,
+									 _("Edit Deck")))),
+    m_vcard_edit_button (manage (new ButtonWithImageLabel (
+									 Gtk::Stock::DND, _("Edit Card")))),
     m_selected_side     (FRONT),
     m_notebook          (manage (new Gtk::Notebook)),
     m_play_progress     (manage (new Gtk::ProgressBar)),
+    m_answer_box_width  (0),
+	m_verify_control    (manage (new VerifyControl (*this))),
     m_bg_white          ("white"),
     m_bg_pink           ("pink"),
     m_bg_black          ("black"),
@@ -347,9 +355,8 @@ DeckPlayer (VDeck& vdeck_)
     m_deck_iter         (m_deck.end ()),         /* empty deck */
     m_is_lesson_learned (false),
     m_is_real_deck      (true),
-    m_answer_box_width  (0),
-	m_verify_control    (manage (new VerifyControl (*this))),
-	m_tab_activated     (false)
+	m_tab_activated     (false),
+	m_initialized       (false)
 {
     trace_with_mask("DeckPlayer::DeckPlayer", GUITRACE);
 
@@ -369,13 +376,12 @@ DeckPlayer (VDeck& vdeck_)
 	 * A desktop Dialog - we can resize and move the window around
 	 */	
     set_resizable (true);
-
+	set_position (Gtk::WIN_POS_CENTER_ALWAYS);
 	Gdk::Rectangle geom = CONFIG->get_deck_player_geometry ();
+	DL ((GRAPP,"Setting size to width=%d, height=%d\n",
+		 geom.get_width (), geom.get_height ()));
 	set_size_request (geom.get_width (), geom.get_height ());
-
-    set_position (Gtk::WIN_POS_CENTER_ALWAYS);
-    set_modal (false);
-
+	set_modal (true);
 	set_transient_for (*MAINWIN);
 
 #endif
@@ -395,74 +401,120 @@ DeckPlayer (VDeck& vdeck_)
     Gtk::HBox* nb_box = manage (new Gtk::HBox (false, 2));
     nb_box->pack_start (*m_notebook, true, true);
 
-    /** Add the notebook
+    /********************
+	 * Add the notebook *
+	 ********************
      */
-    m_notebook->set_tab_pos (Gtk::POS_BOTTOM);
+    m_notebook->set_tab_pos     (Gtk::POS_BOTTOM);
     m_notebook->set_show_border (true);
-    m_notebook->modify_bg   (Gtk::STATE_NORMAL, m_bg_white);
+    m_notebook->modify_bg       (Gtk::STATE_NORMAL, m_bg_white);
+    m_notebook->modify_base     (Gtk::STATE_NORMAL, m_bg_white);
+
+#ifndef IS_HILDON
+
+	/** "The other useful fields are the min_aspect and max_aspect fields; 
+	 *   these contain a width/height ratio as a floating point number. 
+	 *   If a geometry widget is set, the aspect applies to the geometry 
+	 *   widget rather than the entire window. The most common use of these 
+	 *   hints is probably to set min_aspect and max_aspect to the same value, 
+	 *   thus forcing the window to keep a constant aspect ratio. 
+	 */
+	if (CONFIG->keep_deck_player_aspect ()) 
+	{
+		Gdk::Rectangle& rec = CONFIG->get_deck_player_aspect ();
+		gdouble aspect = rec.get_width () * 1.0 / rec.get_height ();
+
+		Gdk::Geometry answer_box_geometry = 
+		{ 
+			geom.get_width(),/*min_width*/ geom.get_height (), /* min_height  */
+			10000,	         /* max_width; */	10000,	       /* max_height  */
+			-1,	             /* base_width */	-1,	           /* base_height */
+			-1,	             /* width_inc  */	-1,	           /* height_inc  */
+			aspect,	         /* min_aspect (width/height) */
+			aspect	         /* max_aspect (width/height) */
+		};
+
+		set_geometry_hints (*this, answer_box_geometry, 
+							Gdk::HINT_ASPECT | Gdk::HINT_MIN_SIZE);
+	}
+#endif
+
+    /***********************************************
+	 *         Fill up notebook with elements      *
+     ***********************************************
+     */
+
+	/*********************
+	 * Front of the card *
+	 *********************
+	 */
+#ifdef IS_HILDON
+    m_front.set_alignment (Gtk::ALIGN_LEFT, Gtk::ALIGN_CENTER);
+    m_front.set_justify   (Gtk::JUSTIFY_CENTER);
+
+#else
+    m_front.set_alignment (0.46, 0.5);
+    m_front.set_justify    (Gtk::JUSTIFY_LEFT);
+
+    m_front.set_selectable ();
+#endif
+
+    m_front.set_use_markup ();
+	m_front.set_line_wrap  (true);
+    m_front.modify_font    (CONFIG->get_question_font ());
+
+    Gtk::Frame* question_frame = manage (new Gtk::Frame());
+    question_frame->set_label_align (0, 0);
+    question_frame->set_shadow_type (Gtk::SHADOW_ETCHED_IN);
+    question_frame->modify_bg       (Gtk::STATE_NORMAL, m_bg_black);
+
+    question_frame->add (m_front);
+
+	Gtk::EventBox* question_evbox = Gtk::manage (new Gtk::EventBox);
+	question_evbox->modify_bg (Gtk::STATE_NORMAL, m_bg_white);
+	question_evbox->add (*question_frame);
+
+    m_notebook->pages ().push_back (
+		Gtk::Notebook_Helpers::TabElem (*question_evbox, _("Front")));
+
+    /********************
+	 * Back of the card *
+	 ********************
+     */
 
     /** VBox holds front, separator, example.
      */
     m_answer_box = manage (new Gtk::VBox (false, 5));
     m_answer_box->set_border_width (5); // How far box is placed from the edges.
 
-#ifndef IS_HILDON
-	if (CONFIG->keep_deck_player_aspect ()) {
-		Gdk::Rectangle& rec = CONFIG->get_deck_player_aspect ();
-		gdouble aspect = rec.get_width () * 1.0 / rec.get_height ();
-		Gdk::Geometry answer_box_geometry = { 
-			 1, /* min_width  */	 1, /* min_height  */
-			-1,	/* max_width; */	-1,	/* max_height  */
-			-1,	/* base_width */	-1,	/* base_height */
-			-1,	/* width_inc  */	-1,	/* height_inc  */
-			aspect,	                    /* min_aspect (width/height) */
-			aspect	                    /* max_aspect (width/height) */
-		};
-		set_geometry_hints (*this, answer_box_geometry, 
-							Gdk::HINT_ASPECT | Gdk::HINT_MIN_SIZE);
-	}
-#endif
-
-    /** Fill up notebook with elements; first front of the card
-     */
-#ifndef IS_HILDON
-    m_front.set_selectable ();
-#endif
-    m_front.set_alignment (0.46, 0.5);
-    m_front.set_justify    (Gtk::JUSTIFY_LEFT);
-    m_front.set_use_markup ();
-	m_front.set_line_wrap  (true);
-    m_front.modify_font    (CONFIG->get_question_font ());
-
-    Gtk::Frame* frame = manage (new Gtk::Frame());
-    frame->set_label_align (0, 0);
-    frame->set_shadow_type (Gtk::SHADOW_ETCHED_IN);
-    frame->modify_bg       (Gtk::STATE_NORMAL, m_bg_black);
-
-    frame->add (m_front);
-
-    m_notebook->pages ().push_back (
-		Gtk::Notebook_Helpers::TabElem (*frame, _("Front")));
-
-    /** Back of the card
-     */
-    /* -separator- */
-    Gtk::HSeparator* separator = manage (new Gtk::HSeparator);
-    separator->modify_bg   (Gtk::STATE_NORMAL, m_bg_pink);
-    separator->modify_fg   (Gtk::STATE_NORMAL, m_bg_pink);
-    separator->modify_base (Gtk::STATE_NORMAL, m_bg_pink);
-
-    /* -back- */
+    /* -Back (the answer)- 
+	 */
+#ifdef IS_HILDON
+    m_back.set_justify    (Gtk::JUSTIFY_CENTER);
+	m_back.set_alignment  (Gtk::ALIGN_LEFT, Gtk::ALIGN_CENTER);
+#else
     m_back.set_justify    (Gtk::JUSTIFY_FILL);
-#ifndef IS_HILDON
     m_back.set_selectable ();
 #endif
+
     m_back.set_use_markup ();
     m_back.modify_font    (CONFIG->get_answer_font ());
     m_back.set_line_wrap  ();
 
-    /* -example- */
-    m_example.set_padding    (6, 0);  /* x - distance between text and 
+	Gtk::EventBox* answer_evbox = Gtk::manage (new Gtk::EventBox);
+	answer_evbox->modify_bg (Gtk::STATE_NORMAL, m_bg_white);
+	answer_evbox->add (m_back);
+
+    /* -Separator- 
+	 */
+    Gtk::HSeparator* separator = manage (new Gtk::HSeparator);
+    separator->modify_fg   (Gtk::STATE_NORMAL, m_bg_pink);
+    separator->modify_bg   (Gtk::STATE_NORMAL, m_bg_pink);
+    separator->modify_base (Gtk::STATE_NORMAL, m_bg_pink);
+
+    /* -Example- 
+	 */
+    m_example.set_padding    (4, 0);  /* x - distance between text and 
 										 right/left edges */
     m_example.set_justify    (Gtk::JUSTIFY_LEFT);
 #ifndef IS_HILDON
@@ -471,33 +523,51 @@ DeckPlayer (VDeck& vdeck_)
     m_example.set_use_markup ();
     m_example.modify_font    (CONFIG->get_example_font ());
     m_example.set_line_wrap  ();
-    m_example.show ();
 
-    m_answer_box->pack_start (m_back,     Gtk::PACK_SHRINK, 5);
-    m_answer_box->pack_start (*separator, Gtk::PACK_SHRINK, 5);
-    m_answer_box->pack_start (m_example,  Gtk::PACK_SHRINK, 5); 
-    m_answer_box->show ();
+	Gtk::EventBox* example_evbox = Gtk::manage (new Gtk::EventBox);
+	example_evbox->modify_bg (Gtk::STATE_NORMAL, m_bg_white);
+	example_evbox->add (m_example);
 
-#ifdef IS_HILDON
+    m_answer_box->pack_start (*answer_evbox,  Gtk::PACK_SHRINK, 5);
+    m_answer_box->pack_start (*separator,     Gtk::PACK_SHRINK, 5);
+    m_answer_box->pack_start (*example_evbox, Gtk::PACK_SHRINK, 5); 
 
+	Gtk::Viewport *viewport =
+		Gtk::manage (new Gtk::Viewport (*manage (new Gtk::Adjustment(0,0,1)),
+										*manage (new Gtk::Adjustment(0,0,1))));
+
+	viewport->set_shadow_type(Gtk::SHADOW_IN);
+	viewport->modify_bg (Gtk::STATE_NORMAL, m_bg_white);
+	
+	viewport->add (*m_answer_box);
+
+	/** Without horizontal AUTOMATIC policy, dialog's resizing
+	 *  by the user doesn't function properly. Instead, the window
+	 *  keeps growing in size every time there is more text to fit then
+	 *  size allows.
+	 */
 	Gtk::ScrolledWindow* scrollw = Gtk::manage (new Gtk::ScrolledWindow);
 
 	scrollw->set_flags (Gtk::CAN_FOCUS);
 	scrollw->set_shadow_type (Gtk::SHADOW_NONE);
-	scrollw->set_policy (Gtk::POLICY_NEVER , Gtk::POLICY_AUTOMATIC);
 	scrollw->property_window_placement ().set_value (Gtk::CORNER_TOP_LEFT);
 
-	scrollw->add (*m_answer_box);
-    m_notebook->pages ().push_back (
-		Gtk::Notebook_Helpers::TabElem (*scrollw, _("Back")));
+#ifdef IS_HILDON
+	scrollw->set_policy (Gtk::POLICY_NEVER , Gtk::POLICY_AUTOMATIC);
 #else
-    m_notebook->pages ().push_back (
-		Gtk::Notebook_Helpers::TabElem (*m_answer_box, _("Back")));
+	scrollw->set_policy (Gtk::POLICY_AUTOMATIC , Gtk::POLICY_AUTOMATIC);
 #endif
 
-    /** Side-panel controls 
+	scrollw->add (*viewport);
+
+    m_notebook->pages ().push_back (
+		Gtk::Notebook_Helpers::TabElem (*scrollw, _("Back")));
+
+    /***********************
+	 * Side-panel controls *
+	 ***********************
      */
-    Gtk::VBox* vbox = manage (new Gtk::VBox (false, 5));
+    Gtk::VBox* vbox = manage (new Gtk::VBox (false, 3));
 
     Gtk::RadioButton::Group group;
     Gtk::RadioButton* radio_fb = NULL;
@@ -539,17 +609,14 @@ DeckPlayer (VDeck& vdeck_)
 
 	/** Fetch out the side history and adjust accordingly
 	 */
-    if (m_is_real_deck) {
-		m_selected_side = CONFIG->get_flipside_history (m_deck.get_name ());
-		if (m_selected_side == FRONT) {
-			radio_fb->set_active ();
-			m_test_line_button->set_sensitive (false);
-		}
-		else {
-			radio_bb->set_active ();
-			m_test_line_button->set_sensitive (true);
-		}
-    }
+	if ((m_selected_side = vdeck_.get_side_selection ()) == FRONT) {
+		radio_fb->set_active ();
+		m_test_line_button->set_sensitive (false);
+	}
+	else {
+		radio_bb->set_active ();
+		m_test_line_button->set_sensitive (true);
+	}
 
     /** Side-panel buttons
      */
@@ -579,7 +646,7 @@ DeckPlayer (VDeck& vdeck_)
     /** Bottom playback controls toolbar panel
      */
     m_playit_xpm = Gdk::Pixbuf::create_from_file (
-		GRAPPDATADIR "/pixmaps/speaker_24.xpm");
+		GRAPPDATDIR "/pixmaps/speaker_24.png");
 	
     Glib::RefPtr<Gtk::IconFactory> icon_factory = Gtk::IconFactory::create ();
 
@@ -592,7 +659,7 @@ DeckPlayer (VDeck& vdeck_)
     Gtk::Stock::add (Gtk::StockItem (DeckPlayer::SAY_IT, _("Say It!")));
 
 	Glib::RefPtr<Gdk::Pixbuf> thumbup_xpm = 
-		Gdk::Pixbuf::create_from_file (GRAPPDATADIR "/pixmaps/thumbup.xpm");
+		Gdk::Pixbuf::create_from_file (GRAPPDATDIR "/pixmaps/thumbup.png");
     Gtk::IconSet thumbup_set;
     Gtk::IconSource thumbup_source;
     thumbup_source.set_pixbuf (thumbup_xpm);
@@ -602,7 +669,7 @@ DeckPlayer (VDeck& vdeck_)
     Gtk::Stock::add (Gtk::StockItem (DeckPlayer::THUMB_UP, _("I know it!")));
 
 	Glib::RefPtr<Gdk::Pixbuf> thumbdown_xpm = 
-		Gdk::Pixbuf::create_from_file (GRAPPDATADIR "/pixmaps/thumbdown.xpm");
+		Gdk::Pixbuf::create_from_file (GRAPPDATDIR "/pixmaps/thumbdown.png");
     Gtk::IconSet thumbdown_set;
     Gtk::IconSource thumbdown_source;
     thumbdown_source.set_pixbuf (thumbdown_xpm);
@@ -703,8 +770,8 @@ DeckPlayer (VDeck& vdeck_)
 		mem_fun (*this, &DeckPlayer::on_close_clicked));
 
 	/**
-	 * Perhaps you should examine signal_realize() instead.
-	 * It is called once.
+	 * Setting resize callbacks. The answer box holds 
+	 * Back/Separator/Example triplet trapped in the scrollwindow.
 	 */
     m_answer_box->signal_size_allocate ().connect (
 		mem_fun (*this, &DeckPlayer::answer_box_size_allocate_cb));
@@ -712,7 +779,7 @@ DeckPlayer (VDeck& vdeck_)
     signal_size_allocate ().connect (
 		mem_fun (*this, &DeckPlayer::size_allocate_cb));
 
-	add_events (Gdk::BUTTON_PRESS_MASK);
+	add_events (Gdk::BUTTON_PRESS_MASK|Gdk::EXPOSURE_MASK);
 
 	/** 'false' argument tells Gtk+ to call on_key_pressed() BEFORE
 		keypress event handling. Returning 'true' from on_key_pressed()
@@ -733,7 +800,7 @@ DeckPlayer (VDeck& vdeck_)
 	/** Grand finale
 	 */
 #ifndef IS_HILDON
-	set_icon_from_file (GRAPPDATADIR "/pixmaps/deckplayer_32x32.png");
+	set_icon_from_file (GRAPPDATDIR "/pixmaps/deckplayer_32x32.png");
 #endif
     set_sensitivity ();
 	sensitize_controls ();
@@ -742,6 +809,8 @@ DeckPlayer (VDeck& vdeck_)
     show_deck ();
     switch_side ();
     show_all_children ();	
+
+	m_initialized = true;
 }
 
 //==============================================================================
@@ -815,7 +884,7 @@ reset ()
  * "Gtk-WARNING **: Failed to set label from markup due to error parsing markup"
  *
  * I'd like ultimately to catch the error and show it in a warning dialog 
- * to the user, but for now I don't see a way of doing it with GTK+.
+ * to the user, but for now I don't see any way of doing it with GTK+.
  */
 void
 DeckPlayer::
@@ -839,13 +908,24 @@ show_card (bool with_adjust_count_)
 		m_front.set_markup ((*m_deck_iter)->get_question ());
 		set_answer_field ((*m_deck_iter)->get_answer ());
 		m_example.set_markup (markup_tildas((*m_deck_iter)->get_example()));
+
+		if ((*m_deck_iter)->get_reversed ()) {
+			m_notebook->set_current_page (BACK);
+		}
     }
 
 	if (with_adjust_count_) {
 		adjust_count ();
 	}
 
+#ifndef IS_HILDON
+	/** Calling set_focus_on_entry() causes crash with Maemo 2.0 when
+	 *  you open CardFile twice in the row. 
+	 *  No clue - investigate later.
+     */
 	m_verify_control->set_focus_on_entry ();
+#endif
+
 }
 
 void
@@ -1014,7 +1094,7 @@ go_next_cb ()
 	    show_card ();
 	}
     /** We flip the side only if user has gone (hopefully)
-     *  through the entier deck.
+     *  through an entire Deck (for *real* Decks only).
      */
     if (m_is_real_deck) {
 		if (m_deck_iter + 1 == m_deck.end () && !m_is_lesson_learned) {	
@@ -1028,7 +1108,7 @@ go_next_cb ()
 	m_verify_control->clear ();
 }
 
-/** This callback is activated only when learning the cards in the
+/** This callback is activated only when learning flashcards in the
     Card Box. We are always at the beginning of the list, moving
     elements to the next box or in case of the last box, moving 
     them to the back of the same list. We proceed in this manner until 
@@ -1085,10 +1165,12 @@ on_radio_selected (SideSelection selected_)
     trace_with_mask("DeckPlayer::on_radio_selected",GUITRACE);
 
     m_selected_side = selected_;
+	m_deck.set_side_selection (m_selected_side);
     switch_side ();
+	m_test_line_button->set_sensitive ((m_selected_side == BACK));
+
 	DL ((GRAPP,"selected = %s\n",
 		 (m_selected_side == BACK ? "BACK" : "FRONT")));
-	m_test_line_button->set_sensitive ((m_selected_side == BACK));
 }
 
 void
@@ -1116,8 +1198,8 @@ on_shuffle_clicked ()
                                Gtk::MESSAGE_QUESTION, 
 							   Gtk::BUTTONS_NONE,
                                true);
-        qm.add_button (_("Yes"), Gtk::RESPONSE_YES);
-        qm.add_button (_("No"), Gtk::RESPONSE_NO);
+        qm.add_button (Gtk::Stock::NO,  Gtk::RESPONSE_NO);
+        qm.add_button (Gtk::Stock::YES, Gtk::RESPONSE_YES);
 		if (qm.run () == Gtk::RESPONSE_YES) {
 			Deck& real_deck = dynamic_cast<Deck&>(m_deck);
 			real_deck.mark_as_modified ();
@@ -1155,14 +1237,18 @@ on_edit_clicked ()
 		dv_modified = dv.is_modified ();
 	}
 
-    DL((GRAPP,"Deck size after edit = %d cards\n", m_deck.size ()));
+    DL((GRAPP,"Deck size after edit = %d cards (modified = %s)\n", 
+		m_deck.size (), (dv_modified ? "TRUE" : "FALSE")));
+
 	sensitize_controls ();
+
     /** 
      * Update viewer - iterators might have become invalid.
      */
-    if (m_deck.size () == 0) {	// Last card removed - clean up
+    if (m_deck.size () == 0) 	// Last card removed - clean up
+	{
 		m_deck_iter = m_deck.end ();
-		DL ((APP,"m_deck_iter = end ()\n"));
+		CARDBOX->mark_as_modified ();
 		show_card ();
 		
 #ifndef IS_HILDON
@@ -1171,9 +1257,11 @@ on_edit_clicked ()
 		on_close_clicked ();
 #endif
     }
-    else {
+    else 
+	{
 		if (dv_modified) {
-			DL ((APP,"Deck's been modified - go to start\n"));
+			DL ((APP,"Deck's been modified - go to the beginning\n"));
+			CARDBOX->mark_as_modified ();
 			show_deck ();
 		}
 		else {
@@ -1215,9 +1303,8 @@ on_verify_clicked ()
 
 	if (m_verify_control->get_state () == VC_ARMED) 
 	{
-		m_verify_control->compare (m_selected_side == FRONT ?
-								   m_back.get_text () :
-								   m_front.get_text ());
+		m_verify_control->compare (get_answer ());
+		auto_pronounce_card (BACK);
 	}
 	else if (m_verify_control->get_state () == VC_VALIDATED) 
 	{
@@ -1299,7 +1386,7 @@ bool
 DeckPlayer::
 on_key_pressed (GdkEventKey* evt_)
 {
-    trace_with_mask("DeckPlayer::on_key_pressed",GUITRACE);
+    trace_with_mask("DeckPlayer::on_key_pressed",KEYIN);
 
 #ifndef IS_HILDON
 	if (evt_->keyval == GDK_F1) {
@@ -1308,7 +1395,7 @@ on_key_pressed (GdkEventKey* evt_)
 #endif
 
 	if (evt_->keyval == GDK_space || evt_->keyval > 0xFF51) {
-		DL ((GRAPP,"Key pressed: 0x%04X, real_deck: %s, state: %s\n", 
+		DL ((KEYIN,"Key pressed: 0x%04X, real_deck: %s, state: %s\n", 
 			 evt_->keyval, (m_is_real_deck ? "Yes" : "No"),
 			 (m_verify_control->get_state() == VC_START ? "START" :
 			  m_verify_control->get_state() == VC_ARMED ? "ARMED" : "VALIDATED")
@@ -1358,9 +1445,8 @@ on_key_pressed (GdkEventKey* evt_)
 			if (evt_->keyval == GDK_Return || 
 				evt_->keyval == GDK_KP_Enter) 
 			{
-				m_verify_control->compare (m_selected_side == FRONT ?
-										   m_back.get_text () :
-										   m_front.get_text ());
+				m_verify_control->compare (get_answer ());
+				auto_pronounce_card (BACK);
 			}
 			else if (evt_->keyval == GDK_space) {
 				return false;
@@ -1394,9 +1480,8 @@ on_key_pressed (GdkEventKey* evt_)
 			if (evt_->keyval == GDK_Return || 
 				evt_->keyval == GDK_KP_Enter) 
 			{
-				m_verify_control->compare (m_selected_side == FRONT ?
-										   m_back.get_text () :
-										   m_front.get_text ());
+				m_verify_control->compare (get_answer ());
+				auto_pronounce_card (BACK);
 			}
 			else if (evt_->keyval == GDK_space) {
 				return false;
@@ -1462,7 +1547,10 @@ void
 DeckPlayer::
 size_allocate_cb (Gtk::Allocation& allocation_)
 {
-//    trace_with_mask("DeckPlayer::size_allocate_cb",GUITRACE);
+    trace_with_mask("DeckPlayer::size_allocate_cb",GEOM);
+
+	DL ((GEOM,"New size change requested: w=%d, h=%d\n", 
+		 allocation_.get_width (), allocation_.get_height ()));
 
 	CONFIG->set_deck_player_geometry (allocation_.get_width (),
 									  allocation_.get_height ());
@@ -1478,11 +1566,11 @@ answer_box_size_allocate_cb (Gtk::Allocation& allocation_)
 {
     trace_with_mask("DeckPlayer::answer_box_size_allocate_cb",GEOM);
 
-    DL((GEOM,"VBox size (w=%d; h=%d)\n", 
+    DL((GEOM,"new answer_box size (w=%d; h=%d)\n", 
 		allocation_.get_width (), allocation_.get_height ()));
 
     if (m_answer_box_width != allocation_.get_width ()) {
-		m_example.set_size_request (allocation_.get_width () - 14,  -1);
+		m_example.set_size_request (allocation_.get_width () - 20,  -1);
 		m_answer_box_width = allocation_.get_width ();
 	}
 }
@@ -1498,9 +1586,28 @@ repaint ()
    m_example.modify_font (CONFIG->get_example_font ());
    m_verify_control->repaint ();
 
+   update_geometry ();
+}
+
+/** Resize only if geometry stored in Configurator differs
+ *  from the actual size;
+ */
+void
+DeckPlayer::
+update_geometry ()
+{
 #ifndef IS_HILDON
-   Gdk::Rectangle geom = CONFIG->get_deck_player_geometry ();
-   resize (geom.get_width (), geom.get_height ());
+	int actual_height = 0;
+	int actual_width = 0;
+
+	Gdk::Rectangle geom = CONFIG->get_deck_player_geometry ();
+	get_size (actual_width, actual_height);
+
+	if (actual_width != geom.get_width () || 
+		actual_height != geom.get_height ())
+	{
+		resize (geom.get_width (), geom.get_height ());
+	}
 #endif
 }
 
@@ -1551,8 +1658,8 @@ markup_tildas (const Glib::ustring& src_)
 		return ret;
 	}
 
-	int i;
-	int b;
+	u_int i;
+	u_int b;
 
 	for (i = 0, b = 0; i < src_.size (); i++) {
 		if (src_[i] == '~') {
@@ -1578,3 +1685,66 @@ markup_is_valid (VCard* card_)
 			Granule::check_markup (card_->get_answer   ()) &&
 			Granule::check_markup (card_->get_example  ()));
 }
+
+/** If card we view is reversed, disregard Deck's orientation. 
+ *  In this case, each card carries its own *sticky* orientation history
+ *  which makes the whole experience of coding and testing maddening!!!
+ */
+void
+DeckPlayer::
+switch_side ()
+{
+    trace_with_mask("DeckPlayer::switch_side", GUITRACE);
+
+	if (m_deck_iter == m_deck.end ()) {
+		m_notebook->set_current_page (m_selected_side);
+		return;
+	}
+
+	if (!m_is_real_deck && (*m_deck_iter)->get_reversed ())
+	{
+		m_notebook->set_current_page (m_selected_side == FRONT ? BACK : FRONT);
+	}
+	else {
+		m_notebook->set_current_page (m_selected_side);
+	}
+
+	auto_pronounce_card (!(*m_deck_iter)->get_reversed () ? FRONT : BACK);
+}
+
+/** We want to have card already painted in the window.
+ */
+void
+DeckPlayer::
+auto_pronounce_card (SideSelection side_)
+{
+    trace_with_mask("DeckPlayer::auto_pronounce_card", GUITRACE);
+
+	if (!m_initialized || m_deck_iter == m_deck.end ()) {
+		return;
+	}
+
+	if (m_selected_side == side_ && CONFIG->auto_pronounce ())
+    {
+		while (Gtk::Main::instance ()->events_pending ()) {
+			Gtk::Main::instance ()->iteration ();
+		}
+
+		m_deck.play_card (m_deck_iter);
+	}
+}
+
+Glib::ustring
+DeckPlayer::
+get_answer ()
+{
+    trace_with_mask("DeckPlayer::get_answer", GUITRACE);
+
+	if ((*m_deck_iter)->get_reversed () || m_selected_side == BACK) {
+		return (m_front.get_text ());
+	}
+	else {
+		return (m_back.get_text ());
+	}
+}
+
