@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //------------------------------------------------------------------------------
-// $Id: DeckPlayer.h,v 1.44 2006/03/19 15:39:44 vlg Exp $
+// $Id: DeckPlayer.h,v 1.49 2006/10/06 02:04:47 vlg Exp $
 //------------------------------------------------------------------------------
 //                            DeckPlayer.h
 //------------------------------------------------------------------------------
@@ -35,7 +35,7 @@
 #include <hildon-widgetsmm/appview.h>
 #endif
 
-#include "Config.h"
+#include "GrappConf.h"
 #include "VDeck.h"
 
 class DeckView;
@@ -64,45 +64,47 @@ class VerifyControl : public Gtk::Table
 public:
 	VerifyControl (DeckPlayer& parent_);
 
-	void set_focus_on_entry (); // { m_entry->grab_focus (); }
-	void compare (const Glib::ustring& answer_);
-	bool matched () const { return m_was_equal; }
-	void clear   ();
+	void    set_focus_on_entry ();
+
+	void    compare (const Glib::ustring& answer_);
+	bool    matched () const { return m_was_equal; }
+	void    clear   ();
 
 	VCState get_state () const;
 	void    set_state (VCState state_);
 
-	void on_entry_changed ();
-	void on_test_line_changed ();
+	void    on_entry_changed ();
+	void    on_test_line_changed ();
 
-	void attach (Gtk::SpinButton* tl_) { m_test_line = tl_; }
-
-	void repaint () { m_entry->modify_font (CONFIG->get_input_font ()); }
+	void    attach (Gtk::SpinButton* tl_) { m_test_line = tl_; }
+	void    repaint () { m_entry->modify_font (CONFIG->get_input_font ()); }
 
 private:
 	static const int LABELS_SZ = 4;
 
-	DeckPlayer&  m_parent;
-	Gtk::Label*  m_report;		// Correct, Incorrect
-	Gtk::EventBox* m_report_box; // Box to hold report label
-	Gtk::Entry*  m_entry;
-	Gtk::Button* m_button;		// 'Check answer', 'Next'
-	Gtk::Label*  m_label [LABELS_SZ];
+	DeckPlayer&      m_parent;
 
-	bool         m_was_equal;	// Keeps result of last compare() op.
-	VCState      m_state;
+	Gtk::Label*      m_report;		      // Correct, Incorrect
+	Gtk::EventBox*   m_report_box;        // Box to hold report label
+	Gtk::Entry*      m_entry;
+	Gtk::Button*     m_button;	          // 'Check answer', 'Next'
+	Gtk::Label*      m_label [LABELS_SZ];
 
-	Gdk::Color   m_red;
-	Gdk::Color   m_green;
-	Gdk::Color   m_white;
-	Gdk::Color   m_black;
+	bool             m_was_equal;         // Keeps result of last compare() op.
+	VCState          m_state;
 
-	Gtk::SpinButton* m_test_line; // Which line to compare for answer.
-	int          m_line_num;	// Number of line to test
+	Gdk::Color       m_red;
+	Gdk::Color       m_green;
+	Gdk::Color       m_white;
+	Gdk::Color       m_black;
+
+	Gtk::SpinButton* m_test_line;         // Which line to compare for answer.
+	int              m_line_num;          // Number of line to test
 };
 
-/**
- * Class DeckPlayer 
+/**=============================================================================
+ *                              Class DeckPlayer 
+ **=============================================================================
  */
 #ifdef IS_HILDON 
 class DeckPlayer : public Hildon::AppView
@@ -176,6 +178,9 @@ public:
 	*/
 	void reset ();
 
+	/// Reset geometry from Configurator
+	void update_geometry ();
+
 	/** Take into account changed Preferences: adjust geometry
 		and/or repaint the selected card.
 	*/
@@ -194,6 +199,9 @@ public:
     /// Flip the Card
 	void flip_card ();	
 
+	/// Auto-pronounce word on next/flip action
+	void auto_pronounce_card (SideSelection side_);
+
 	SideSelection get_selected_size () const { return m_selected_side; }
 
 	/** Remove all newlines and shrink all whitespace.
@@ -224,6 +232,10 @@ private:
 	 */
 	void show_card (bool with_adjust_count_ = true);	
 
+	/** What should be considered an answer?
+	 */
+	Glib::ustring get_answer ();
+
 	/** Adjust justification of the 'Answer' field depending on
 	 *  wether it is a single line or multiline
 	 */
@@ -250,6 +262,7 @@ private:
 #ifdef IS_HILDON
 	bool              m_hildon_fullscreen;
 #endif
+
 	Gtk::Button*      m_close_button;
 	Gtk::Button*      m_shuffle_button;
 	Gtk::Button*      m_edit_button;         // View/Edit a Deck
@@ -293,19 +306,13 @@ private:
 	Glib::RefPtr<Gtk::ActionGroup> m_actgroup;
 
 	bool m_tab_activated;		// Leave Tab+Spacebar/Enter event to Dialog
+	bool m_initialized;	        // Has the object been fully constructed?
 
 #ifdef IS_HILDON
-	Gtk::VBox*       m_vbox;
+	Gtk::VBox*        m_vbox;
 #endif
 };
 
-inline void
-DeckPlayer::
-switch_side ()
-{
-	m_notebook->set_current_page (m_selected_side);
-}
-
 inline void
 DeckPlayer::
 flip_card ()
@@ -364,6 +371,4 @@ check_F8_key (gint keyval_)
 #endif
 }
 
-
-
 #endif /* DECK_PLAYER_H */
