@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //------------------------------------------------------------------------------
-// $Id: DeckView.cpp,v 1.41 2006/03/19 15:39:44 vlg Exp $
+// $Id: DeckView.cpp,v 1.48 2006/09/30 20:42:27 vlg Exp $
 //------------------------------------------------------------------------------
 //                            DeckView.cpp
 //------------------------------------------------------------------------------
@@ -33,6 +33,7 @@
 #include "DeckView.h"
 #include "DeckInfo.h"
 #include "ButtonWithImageLabel.h"
+#include "Intern.h"
 
 using sigc::bind;
 using sigc::mem_fun;
@@ -41,27 +42,24 @@ DeckView::
 DeckView (VDeck& deck_, int selected_row_) 
 		:
 #ifdef IS_HILDON
-	Hildon::AppView ("Deck View"),
+	Hildon::AppView    ("Deck View"),
 #endif
-	m_add_button    (manage (new Gtk::Button (Gtk::StockID ("gtk-add")))),
-	m_edit_button   (
-		manage (new ButtonWithImageLabel (Gtk::Stock::DND, "Edit"))),
-	m_delete_button (manage (new Gtk::Button (Gtk::StockID("gtk-delete")))),
-	m_save_deck_button (
-		manage (new ButtonWithImageLabel(Gtk::Stock::SAVE,"Save Deck"))),
-	m_label_count   (manage (new Gtk::Label ("0 cards"))),
-	m_more_button   (
-		manage (new ButtonWithImageLabel (
-					Gtk::Stock::DIALOG_INFO, "Deck Info"))),
-	m_card_down_button (
-		manage (new Gtk::Button (Gtk::Stock::GO_DOWN))),
-	m_card_up_button (
-		manage (new Gtk::Button (Gtk::Stock::GO_UP))),
-	m_add_card_to_cardbox (
-		manage (new ButtonWithImageLabel (Gtk::Stock::UNDO, "Add to Box 1 "))),
-	m_deck           (deck_),
-	m_row_idx        (selected_row_),
-	m_modified       (false)
+	m_add_button       (manage (new Gtk::Button (Gtk::StockID ("gtk-add")))),
+	m_edit_button      (manage (new ButtonWithImageLabel (
+									Gtk::Stock::DND, "Edit"))),
+	m_delete_button    (manage (new Gtk::Button (Gtk::StockID("gtk-delete")))),
+	m_save_deck_button (manage (new ButtonWithImageLabel(
+									Gtk::Stock::SAVE,"Save Deck"))),
+	m_more_button      (manage (new ButtonWithImageLabel (
+									Gtk::Stock::DIALOG_INFO, "Deck Info"))),
+	m_card_up_button   (manage (new Gtk::Button (Gtk::Stock::GO_UP))),
+	m_card_down_button (manage (new Gtk::Button (Gtk::Stock::GO_DOWN))),
+	m_add_card_to_cardbox (manage (new ButtonWithImageLabel (
+									   Gtk::Stock::UNDO, "Add to Box 1 "))),
+	m_label_count      (manage (new Gtk::Label ("0 cards"))),
+	m_deck             (deck_),
+	m_row_idx          (selected_row_),
+	m_modified         (false)
 {
 	trace_with_mask("DeckView::DeckView", GUITRACE);
 
@@ -76,13 +74,12 @@ DeckView (VDeck& deck_, int selected_row_)
 	 */
     m_vbox = manage (new Gtk::VBox (false, 2));
 	add (*m_vbox);
-
 	set_size_request (672, 396);
-//	set_transient_for (*HILDONAPPWIN);
 #else
 	set_resizable(true);
 	set_position (Gtk::WIN_POS_CENTER_ALWAYS);
 	set_size_request (633, 410);	// width; height 
+	set_modal (true);
 	set_transient_for (*MAINWIN);
 #endif
 
@@ -158,15 +155,24 @@ DeckView (VDeck& deck_, int selected_row_)
 	m_scrollwin->set_shadow_type (Gtk::SHADOW_ETCHED_OUT);
 	m_scrollwin->set_policy (Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
 
+	Glib::ustring qstr;
 	Gtk::TreeModel::Row row;
-	Card* card;
 	m_list_store_ref = Gtk::ListStore::create (m_columns);
 	VDeck::cardlist_iterator iter = m_deck.begin ();
 
-	while (iter != m_deck.end ()) {
+	while (iter != m_deck.end ()) 
+	{
 		row = *m_list_store_ref->append ();
-		row [m_columns.m_front_row] = 
-			Granule::trim_multiline ((*iter)->get_question ());
+		qstr = Granule::trim_multiline ((*iter)->get_question ());
+
+		if ((*iter)->get_reversed ()) {
+			row [m_columns.m_front_row] = 
+				"<span foreground=\"red\">" + qstr + "</span>";
+		}
+		else {
+			row [m_columns.m_front_row] = qstr;
+		}
+
 		row [m_columns.m_back_row ] = 
 			Granule::trim_multiline ((*iter)->get_answer ());
 		row [m_columns.m_card     ] = *iter;
@@ -232,7 +238,7 @@ DeckView (VDeck& deck_, int selected_row_)
 		mem_fun (*this, &DeckView::on_column_sorted));
 
 	m_tree_sel_ref = m_tree_view.get_selection ();
-	m_tree_sel_ref->set_mode (Gtk::SELECTION_SINGLE);
+	m_tree_sel_ref->set_mode (Gtk::SELECTION_MULTIPLE);
 	m_tree_sel_ref->signal_changed ().connect (
 		mem_fun (*this, &DeckView::on_card_selected));
 
@@ -306,7 +312,7 @@ DeckView (VDeck& deck_, int selected_row_)
 	set_win_name ();
 
 #ifndef IS_HILDON
-	set_icon_from_file (GRAPPDATADIR "/pixmaps/deckview_32x32.png");
+	set_icon_from_file (GRAPPDATDIR "/pixmaps/deckview_32x32.png");
 #endif
 
 }
@@ -377,12 +383,13 @@ on_card_selected ()
 {
 	trace_with_mask("DeckView::on_card_selected", GUITRACE);
 
-	Gtk::TreeModel::iterator iter = m_tree_sel_ref->get_selected ();
-	if (!iter) {
+	pathlist_t pathlist = m_tree_sel_ref->get_selected_rows ();
+	pathlist_iterator_t iter = pathlist.begin ();
+
+	if (iter == pathlist.end ()) {
 		return;
 	}
-	Gtk::TreeModel::Path path = m_list_store_ref->get_path (iter);
-	m_row_idx = *(path.get_indices ().begin ());
+	m_row_idx = *((*iter).get_indices ().begin ());
 	DL((GRAPP,"Row selected : %d\n", m_row_idx));
 }
 
@@ -395,11 +402,14 @@ add_new_card (Card* card_)
 	m_deck.push_back (card_);
 
 	Gtk::TreeModel::Row row = *m_list_store_ref->append ();
+
 	row [m_columns.m_front_row] = 
 		Granule::trim_multiline (card_->get_question ());
 	row [m_columns.m_back_row ] = 
 		Granule::trim_multiline (card_->get_answer ());
 	row [m_columns.m_card     ] = card_;
+
+	m_tree_sel_ref->unselect_all ();
 	m_tree_sel_ref->select (row);
 	set_cards_count ();
 	autoscroll_down ();
@@ -427,7 +437,7 @@ on_add_clicked ()
 		DL((GRAPP,"Text entry syntax error (wrong markup?)\n"));
 	}
 	else {						
-		DL((GRAPP|ASSA::ERROR, "Unexpected response = %d, "
+		DL((GRAPP|ASSA::ASSAERR, "Unexpected response = %d, "
 			"see /path/to/gtkmm/dialog.h for details.\n", ret));
 	}
 }
@@ -470,40 +480,108 @@ DeckView::
 on_delete_clicked ()
 {
 	trace_with_mask("DeckView::on_delete_clicked", GUITRACE);
+
 	if (m_row_idx < 0) {
 		Gtk::MessageDialog em ("The deck is empty!",Gtk::MESSAGE_ERROR);
 		em.run ();
 		return;
 	}
-	/** Delete card from the deck. We can be looking at either Deck
-		or CardDeck. Either way the card is completely deleted from
-		the Deck (even via CardDeck->VCard->CardRef->Deck route).
+
+	/** Get a list of all selected rows. Convert rows into TreePaths.
+	 *  Convert further TreePaths into TreeRowReferences.
 	 */
-	Gtk::TreeModel::Row row = *m_list_store_ref->children ()[m_row_idx];
-	VCard* vcard = row [m_columns.m_card];
-	m_deck.erase (vcard);
+	pathlist_t pathlist = m_tree_sel_ref->get_selected_rows ();
+
+	if (pathlist.size () == 0) {
+		Gtk::MessageDialog em ("Nothing is selected",Gtk::MESSAGE_ERROR);
+		em.run ();
+		return;
+	}
+
+	{
+		Gtk::MessageDialog qm (
+			_("Are you shure you want\nto delete selected card(s)?"),
+			false,
+			Gtk::MESSAGE_QUESTION, 
+			Gtk::BUTTONS_NONE,
+			true);
+		qm.add_button (Gtk::Stock::NO,  Gtk::RESPONSE_NO);
+		qm.add_button (Gtk::Stock::YES, Gtk::RESPONSE_YES);
+
+		if (qm.run () == Gtk::RESPONSE_NO) {
+			return;
+		}
+	}
+
+	/** Convert references to paths
+	 */
+	std::vector<Gtk::TreeRowReference> selected_refs;
+	pathlist_iterator_t iter = pathlist.begin ();
+	while (iter != pathlist.end ()) 
+	{
+		selected_refs.push_back(Gtk::TreeRowReference(m_list_store_ref, *iter));
+		iter++;
+	}
+
+	VCard* vcardp;
+	vector<VCard*> vcards_list;
+	vector<VCard*>::iterator vcards_iter;
+
+	Gtk::TreeModel::iterator dead_row_iter;
+	std::vector<Gtk::TreeRowReference>::reverse_iterator riter;
+
+	/** Delete from the model first using paths (safe).
+	 */
+	riter = selected_refs.rbegin ();
+	while (riter != selected_refs.rend ()) 
+	{
+		dead_row_iter = m_list_store_ref->get_iter (riter->get_path ());
+		Gtk::TreeModel::Row row = *dead_row_iter;
+		vcardp = row [m_columns.m_card];
+		vcards_iter = std::find (vcards_list.begin (), 
+								 vcards_list.end (), 
+								 vcardp);
+		if (vcards_iter == vcards_list.end ()) {
+			DL ((GRAPP,"Adding '%s' to vcards_list\n", 
+				 vcardp->get_question ().c_str ()));
+			vcards_list.push_back (vcardp);
+		}
+
+		m_list_store_ref->erase (dead_row_iter);
+
+		riter++;
+	}
+
+	/** Delete from the CardBox
+	 */
+	vcards_iter = vcards_list.begin ();
+	while (vcards_iter != vcards_list.end ()) {
+		DL ((GRAPP,"Erasing '%s'\n", 
+			 (*vcards_iter)->get_question ().c_str ()));
+		m_deck.erase (*vcards_iter);
+		vcards_iter++;
+	}
+
+	/** Reposition selected row and update counters.
+	 */
+	if (m_row_idx == m_deck.size ()) {
+		m_row_idx--;
+	}
+
+	if (m_row_idx >= 0) {
+		children_t children = m_list_store_ref->children();
+		chiter_t chiter = children [m_row_idx];
+		Gtk::TreeModel::Row row = *chiter;
+		m_tree_sel_ref->select (row);
+		set_cards_count ();
+		autoscroll_down ();
+	}
 
 	Deck* real_deck = NULL;
 	if ((real_deck = dynamic_cast<Deck*>(&m_deck)) != NULL) {
 		real_deck->mark_as_modified (); 
 	}
 
-	/** Delete card from the view list
-	 */
-	children_t children = m_list_store_ref->children();
-	chiter_t chiter = children [m_row_idx];
-	m_list_store_ref->erase (chiter);
-
-	if (m_row_idx == m_deck.size ()) {
-		m_row_idx--;
-	}
-	if (m_row_idx >= 0) {
-		chiter = children [m_row_idx];
-		Gtk::TreeModel::Row row = *chiter;
-		m_tree_sel_ref->select (row);
-		set_cards_count ();
-		autoscroll_down ();
-	}
 	m_modified = true;
 }
 
@@ -555,7 +633,8 @@ autoscroll_down ()
     double pg_sz     = adj->get_page_size ();
 	double last_page = adj->get_upper () - pg_sz;
                                                                                 
-    if (((m_row_idx + 1) * prh - value) > pg_sz) {
+    if (((m_row_idx + 1) * prh - value) > pg_sz) 
+	{
 		if ((tmp = value + pg_inc/2) > last_page) {
 			adj->set_value (last_page);
 		}
@@ -654,9 +733,30 @@ on_add_card_to_cardbox ()
 		return;
 	}
 
-	VDeck::cardlist_iterator card_iter = m_deck.begin () + m_row_idx;
-	CARDBOX->add_card_to_box (
-		new CardRef (**card_iter, dynamic_cast<Deck&>(m_deck)));
+	int row_idx;
+
+	VDeck::cardlist_iterator card_iter;
+	cardlist_t card_list;
+	pathlist_t pathlist = m_tree_sel_ref->get_selected_rows ();
+
+	if (pathlist.size () == 0) {
+		Gtk::MessageDialog em ("Nothing is selected",Gtk::MESSAGE_ERROR);
+		em.run ();
+		return;
+	}
+
+	pathlist_iterator_t iter = pathlist.begin ();
+	while (iter != pathlist.end ()) 
+	{
+		row_idx = *((*iter).get_indices ().begin ());
+		card_iter = m_deck.begin () + row_idx;
+		card_list.push_back (new CardRef (**card_iter, 
+										  dynamic_cast<Deck&>(m_deck)));
+		iter++;
+	}
+
+	CARDBOX->add_cards_to_box (card_list);
+	m_modified = true;
 }
 
 /**
@@ -677,8 +777,7 @@ sort_compare_impl (const Gtk::TreeModel::iterator& a_,
 	gchar* txt_b;
 	gchar* pa;
 	gchar* pb;
-	int ret;
-	int i;
+	int    ret;
 
 	const Gtk::TreeModel::Row row_a = *a_;
 	const Gtk::TreeModel::Row row_b = *b_;
