@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //------------------------------------------------------------------------------
-// $Id: DeckView.h,v 1.23 2006/03/19 15:39:44 vlg Exp $
+// $Id: DeckView.h,v 1.26 2006/09/21 01:25:20 vlg Exp $
 //------------------------------------------------------------------------------
 //                            DeckView.h
 //------------------------------------------------------------------------------
@@ -110,9 +110,24 @@ private:
 	void set_cards_count ();
 	void set_win_name ();
 
+	/** De-activate action buttons depending on the type 
+	    of the underlying VDeck object.
+	*/
+	void set_sensitivity ();
+
+	void autoscroll_up   ();
+	void autoscroll_down ();
+
+#ifdef IS_HILDON
+	Gtk::VBox* get_vbox () { return m_vbox; }
+#endif
+
 private:
 	typedef Gtk::TreeModel::Children children_t;
 	typedef children_t::iterator chiter_t;
+	typedef std::vector<Gtk::TreeModel::Path> pathlist_t;
+	typedef std::vector<Gtk::TreeModel::Path>::iterator pathlist_iterator_t;
+	typedef std::vector<CardRef*> cardlist_t;
 
 	struct ModelColumns : public Gtk::TreeModelColumnRecord
 	{
@@ -126,9 +141,6 @@ private:
 			}
 	};
 
-	void autoscroll_up ();
-	void autoscroll_down ();
-
 	const ModelColumns   m_columns;
 	Gtk::TreeView        m_tree_view;
 	Gtk::ScrolledWindow* m_scrollwin;
@@ -139,15 +151,6 @@ private:
     /// Selection in current TreeView
 	Glib::RefPtr<Gtk::TreeSelection> m_tree_sel_ref; 
 
-	/** De-activate action buttons depending on the type 
-	    of the underlying VDeck object.
-	*/
-	void set_sensitivity ();
-
-#ifdef IS_HILDON
-	Gtk::VBox* get_vbox () { return m_vbox; }
-#endif
-
 private:
 	Gtk::Button* m_add_button;
 	Gtk::Button* m_edit_button;
@@ -161,12 +164,12 @@ private:
 
 	Gtk::Label*  m_label_count;
 
-	VDeck& m_deck;
-	int    m_row_idx;			// The row selected in the model
-	bool   m_modified;			// Has Deck been modified to invalidate
+	VDeck&       m_deck;
+	int          m_row_idx;		// The row selected in the model
+	bool         m_modified;	// Has Deck been modified to invalidate
 
 #ifdef IS_HILDON
-	Gtk::VBox*       m_vbox;
+	Gtk::VBox*   m_vbox;
 #endif
 
 };
