@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //------------------------------------------------------------------------------
-// $Id: EditControls.cpp,v 1.6 2006/02/27 01:15:20 vlg Exp $
+// $Id: EditControls.cpp,v 1.7 2006/06/14 02:34:38 vlg Exp $
 //------------------------------------------------------------------------------
 //                            EditControls.cpp
 //------------------------------------------------------------------------------
@@ -45,52 +45,52 @@ EditControls (Glib::RefPtr<Gtk::TextBuffer> tb_ref_)
 	widget_names.push_back ("TextPaste");
 
 	pixbuf_ref = Gdk::Pixbuf::create_from_file (
-		GRAPPDATADIR "pixmaps/text_bold.png");
+		GRAPPDATDIR "pixmaps/text_bold.png");
 	m_icon_list.push_back (icon_t ("bold", 
 								   manage (new Image (pixbuf_ref))));
 
 	pixbuf_ref = Gdk::Pixbuf::create_from_file (
-		GRAPPDATADIR "pixmaps/text_italic.png");
+		GRAPPDATDIR "pixmaps/text_italic.png");
 	m_icon_list.push_back (icon_t ("italic", 
 								   manage (new Image (pixbuf_ref))));
 
 	pixbuf_ref = Gdk::Pixbuf::create_from_file (
-		GRAPPDATADIR "pixmaps/text_underlined.png");
+		GRAPPDATDIR "pixmaps/text_underlined.png");
 	m_icon_list.push_back (icon_t ("underline", 
 								   manage (new Image (pixbuf_ref))));
 
 	pixbuf_ref = Gdk::Pixbuf::create_from_file (
-		GRAPPDATADIR "pixmaps/text_monospaced.png");
+		GRAPPDATDIR "pixmaps/text_monospaced.png");
 	m_icon_list.push_back (icon_t ("monospace", 
 								   manage (new Image (pixbuf_ref))));
 
 	pixbuf_ref = Gdk::Pixbuf::create_from_file (
-		GRAPPDATADIR "pixmaps/text_subscript.png");
+		GRAPPDATDIR "pixmaps/text_subscript.png");
 	m_icon_list.push_back (icon_t ("subscript", 
 								   manage (new Image (pixbuf_ref))));
 
 	pixbuf_ref = Gdk::Pixbuf::create_from_file (
-		GRAPPDATADIR "pixmaps/text_superscript.png");
+		GRAPPDATDIR "pixmaps/text_superscript.png");
 	m_icon_list.push_back (icon_t ("superscript", 
 								   manage (new Image (pixbuf_ref))));
 
 	pixbuf_ref = Gdk::Pixbuf::create_from_file (
-		GRAPPDATADIR "pixmaps/text_big.png");
+		GRAPPDATDIR "pixmaps/text_big.png");
 	m_icon_list.push_back (icon_t ("big", 
 								   manage (new Image (pixbuf_ref))));
 
 	pixbuf_ref = Gdk::Pixbuf::create_from_file (
-		GRAPPDATADIR "pixmaps/text_small.png");
+		GRAPPDATDIR "pixmaps/text_small.png");
 	m_icon_list.push_back (icon_t ("small", 
 								   manage (new Image (pixbuf_ref))));
 
 	pixbuf_ref = Gdk::Pixbuf::create_from_file (
-		GRAPPDATADIR "pixmaps/text_remove_tags.png");
+		GRAPPDATDIR "pixmaps/text_remove_tags.png");
 	m_icon_list.push_back (icon_t ("remove_tags", 
 								   manage (new Image (pixbuf_ref))));
 
 	pixbuf_ref = Gdk::Pixbuf::create_from_file (
-		GRAPPDATADIR "pixmaps/text_paste.png");
+		GRAPPDATDIR "pixmaps/text_paste.png");
 	m_icon_list.push_back (icon_t ("paste", 
 								   manage (new Image (pixbuf_ref))));
 
