@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //------------------------------------------------------------------------------
-// $Id: GeometryPrefs.cpp,v 1.10 2006/03/19 15:39:44 vlg Exp $
+// $Id: GeometryPrefs.cpp,v 1.11 2006/06/14 02:34:38 vlg Exp $
 //------------------------------------------------------------------------------
 //                            GeometryPrefs.cpp
 //------------------------------------------------------------------------------
@@ -21,7 +21,7 @@
 #include <gtkmm/scrolledwindow.h>
 
 #include "Granule-main.h"
-#include "Config.h"
+#include "GrappConf.h"
 #include "PrefWindow.h"
 #include "GeometryPrefs.h"
 
