@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //------------------------------------------------------------------------------
-// $Id: PrefWindow.cpp,v 1.9 2006/03/19 15:39:44 vlg Exp $
+// $Id: PrefWindow.cpp,v 1.12 2006/09/05 03:25:29 vlg Exp $
 //------------------------------------------------------------------------------
 //                            PrefWindow.h
 //------------------------------------------------------------------------------
@@ -16,13 +16,15 @@
 //
 //------------------------------------------------------------------------------
 
-#include "Config.h"
+#include "GrappConf.h"
 #include "PrefWindow.h"
 
 PrefWindow::
-PrefWindow (Gtk::Widget& parent_) :
+PrefWindow (Gtk::Widget& parent_) 
+	:
     MyPropertyBox      ("Preferences"),
 	m_general_pref     (*this),
+	m_appearance_pref  (*this),
 	m_scheduling_pref  (*this),
 	m_sound_pref       (*this),
 	m_geometry_pref    (*this),
@@ -34,22 +36,25 @@ PrefWindow (Gtk::Widget& parent_) :
 #ifdef IS_HILDON
 	set_size_request (672, -1);
 #else
+	set_size_request (520, 380);
 	set_transient_for (dynamic_cast<Gtk::Window&>(parent_));
 #endif
 
 	/** Show all
 	 */
-	m_general_pref.show_all ();
+	m_general_pref.show_all    ();
+	m_appearance_pref.show_all ();
 	m_scheduling_pref.show_all ();
-	m_sound_pref.show_all ();
-	m_geometry_pref.show_all ();
+	m_sound_pref.show_all      ();
+	m_geometry_pref.show_all   ();
 	
 	/** Pack all
 	 */
-	append_page (m_general_pref,    "General");
+	append_page (m_general_pref,    "General"   );
+	append_page (m_appearance_pref, "Appearance");
 	append_page (m_scheduling_pref, "Scheduling");
-	append_page (m_sound_pref,      "Sound");
-	append_page (m_geometry_pref,   "Geometry");
+	append_page (m_sound_pref,      "Sound"     );
+	append_page (m_geometry_pref,   "Geometry"  );
 }
 
 bool
@@ -64,10 +69,11 @@ run ()
 
 	/** Fill values of all pages with fresh values from Config
 	 */
-	m_general_pref.load_from_config ();
+	m_general_pref.load_from_config    ();
+	m_appearance_pref.load_from_config ();
 	m_scheduling_pref.load_from_config ();
-	m_sound_pref.load_from_config ();
-	m_geometry_pref.load_from_config ();
+	m_sound_pref.load_from_config      ();
+	m_geometry_pref.load_from_config   ();
 	
 	m_ignore_callbacks = false;
 	m_apply_button->set_sensitive (false);
@@ -90,8 +96,9 @@ apply_impl ()
 {
     trace_with_mask("PrefWindow::apply_impl",GUITRACE);
 
-	m_general_pref.save_to_config ();
+	m_general_pref.save_to_config    ();
+	m_appearance_pref.save_to_config ();
 	m_scheduling_pref.save_to_config ();
-	m_sound_pref.save_to_config ();
-	m_geometry_pref.save_to_config ();
+	m_sound_pref.save_to_config      ();
+	m_geometry_pref.save_to_config   ();
 }
