@@ -2,7 +2,7 @@
 //------------------------------------------------------------------------------
 //                              TopMenuBar.cpp
 //------------------------------------------------------------------------------
-// $Id: TopMenuBar.cpp,v 1.25 2006/03/19 15:39:44 vlg Exp $
+// $Id: TopMenuBar.cpp,v 1.27 2006/09/21 01:25:20 vlg Exp $
 //------------------------------------------------------------------------------
 //  Copyright (c) 2004 by Vladislav Grinchenko
 //
@@ -22,7 +22,7 @@
 #include "TopMenuBar.h"
 #include "MainWindow.h"
 #include "DeckManager.h"
-#include "Config.h"
+#include "GrappConf.h"
 
 using namespace ASSA;
 using sigc::bind;
@@ -244,8 +244,8 @@ refill_history_list ()
     file_list.clear ();
 
 	std::string underscored;
-    const Config::DHList& dh_list = CONFIG->get_document_history ();
-    Config::DHList_CIter cit = dh_list.begin ();
+    const GrappConf::DHList& dh_list = CONFIG->get_document_history ();
+    GrappConf::DHList_CIter cit = dh_list.begin ();
 
 	while (cit != dh_list.end ()) {
 		if ((*cit).find ("_") == std::string::npos) {
@@ -275,7 +275,7 @@ TopMenuBar::
 get_menu_bar ()
 {
     if (m_menu_bar == NULL) {
-		DL((ASSA::ERROR, 
+		DL((ASSA::ASSAERR, 
 			"TopMenuBar::get_menu_bar() : m_menu_bar uninitialized!\n"));
 		Assure_exit (false);
     }
@@ -325,6 +325,10 @@ set_sensitivity (WMState state_)
 		m_exit       ->set_sensitive (true ); // exit
 		m_play_in    ->set_sensitive (true ); // play in CardBox
 		break;
+
+	case Finish:
+        /* do-nothing */
+		break;
     }
 }
 
